package Util;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import java.util.ArrayList;
import java.util.List;

import tads.ufpr.pr.br.backstage.Instrumento;
import tads.ufpr.pr.br.backstage.Message;
import tads.ufpr.pr.br.backstage.Usuario;

public class DatabaseHandler extends SQLiteOpenHelper {

    private static final int    DATABASE_VERSION = 1;
    private static final String DATABASE_NAME    = "backstageDB";
    private static final String TABLE_USER       = "usuario";
    private static final String TABLE_UINST      = "userInstrumentos";
    private static final String TABLE_FOLLOW     = "seguindo";
    private static final String TABLE_LMUPDATE   = "lastMsgUpdate";
    private static final String TABLE_MESSAGES   = "messages";

    //Fields
    private static final String KEY_ID      = "id";
    private static final String KEY_NAME    = "name";
    private static final String KEY_EMAIL   = "email";
    private static final String KEY_DATE    = "date";
    private static final String KEY_PASS    = "password";
    private static final String KEY_IMG     = "image";
    private static final String KEY_ABOUT   = "about";
    private static final String KEY_STYLES  = "styles";
    private static final String KEY_INFL    = "influences";
    private static final String KEY_LAT     = "latitude";
    private static final String KEY_LONG    = "longitude";
    private static final String KEY_CITY    = "city";
    private static final String KEY_STATE   = "state";
    private static final String KEY_PAIS    = "pais";
    private static final String KEY_TYPE    = "type";
    private static final String KEY_LEVEL   = "level";

    public DatabaseHandler(Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    @Override
    public void onCreate(SQLiteDatabase db) {

        String CREATE_USER_TABLE = "CREATE TABLE " + TABLE_USER + " ( "
                + KEY_ID    + " INTEGER PRIMARY KEY, "          + KEY_NAME   + " TEXT, "
                + KEY_EMAIL + " TEXT, " + KEY_DATE  + " TEXT, " + KEY_PASS   + " TEXT, "
                + KEY_IMG   + " TEXT, " + KEY_ABOUT + " TEXT, " + KEY_STYLES + " TEXT, "
                + KEY_INFL  + " TEXT, " + KEY_LAT   + " DOUBLE, " + KEY_LONG + " DOUBLE, "
                + KEY_CITY  + " TEXT, " + KEY_STATE + " TEXT, "   + KEY_PAIS + " TEXT )";

        String CREATE_FOLLOW_TABLE = "CREATE TABLE " + TABLE_FOLLOW + " ( "
                + KEY_ID    + " INTEGER PRIMARY KEY, "          + KEY_NAME   + " TEXT, "
                + KEY_IMG   + " TEXT, " + KEY_CITY  + " TEXT )";

        String CREATE_USER_INST_TABLE =  "CREATE TABLE " + TABLE_UINST + " ( "
                + KEY_ID    + " INTEGER PRIMARY KEY, " + KEY_TYPE + " INTEGER, "
                + KEY_NAME  + " TEXT, " + KEY_LEVEL + " INTEGER )";

        String CREATE_LM_TABLE = "CREATE TABLE " + TABLE_LMUPDATE + " ( "
                + KEY_ID    + " INTEGER PRIMARY KEY, "
                + KEY_DATE  + " TEXT )";

        String CREATE_TABLE_MESSAGES = "CREATE TABLE " + TABLE_MESSAGES + " ( "
                + "chatID INTEGER NOT NULL, userID INTEGER NOT NULL, msgID INTEGER NOT NULL,"
                + " texto TEXT, date TEXT, visto TEXT, PRIMARY KEY ( chatID, userID, msgID ))";

        String CREATE_TABLE_USERS = "CREATE TABLE USERS ( "
                + KEY_ID    + " INTEGER PRIMARY KEY, " + KEY_NAME   + " TEXT, "
                + KEY_IMG   + " TEXT, " + KEY_CITY  + " TEXT )";


        db.execSQL(CREATE_USER_TABLE);
        db.execSQL(CREATE_USER_INST_TABLE);
        db.execSQL(CREATE_FOLLOW_TABLE);
        db.execSQL(CREATE_LM_TABLE);
        db.execSQL(CREATE_TABLE_MESSAGES);
        db.execSQL(CREATE_TABLE_USERS);
        db.execSQL("CREATE TABLE CHATUSER ( chatID INTEGER NOT NULL, id INTEGER NOT NULL )");

    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {

        db.execSQL("DROP TABLE IF EXISTS " + TABLE_USER);
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_UINST);
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_FOLLOW);
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_LMUPDATE);
        db.execSQL("DROP TABLE IF EXISTS " + "USERS");

        onCreate(db);

    }

    public void addUser(Usuario u){

        SQLiteDatabase db = this.getWritableDatabase();
        ContentValues values = new ContentValues();

        try {

            String estilos = "", influencias = "";

            if ( u.getEstilos() != null && !u.getEstilos().isEmpty() ){
                for ( String s : u.getEstilos() ){
                    estilos = estilos + s + ",";
                }
            }

            if ( u.getInfluencias() != null && !u.getInfluencias().isEmpty() ){
                for ( String s : u.getInfluencias() ){
                    influencias = influencias + s + ",";
                }
            }

            values.put(KEY_ID,     u.getId());         // User ID
            values.put(KEY_NAME,   u.getNome());       // User Name
            values.put(KEY_EMAIL,  u.getEmail());      // User Email
            values.put(KEY_DATE,   u.getDtNasc());     // User Birth
            values.put(KEY_PASS,   u.getSenha());      // User Password
            values.put(KEY_IMG,    u.getImagem());     // User Image
            values.put(KEY_ABOUT,  u.getSobre());      // User About
            values.put(KEY_STYLES, estilos);           // User Styles
            values.put(KEY_INFL,   influencias);       // User Influences
            values.put(KEY_LAT,    u.getLatitude());   // User Lat
            values.put(KEY_LONG,   u.getLongitude());  // User Long
            values.put(KEY_CITY,   u.getCidade());     // User City
            values.put(KEY_STATE,  u.getEstado());     // User State
            values.put(KEY_PAIS,   u.getPais());       // User Country

            db.insert(TABLE_USER, null, values);
            db.close();

            addInstrument(u);
        }
        catch ( Exception e ){
            e.printStackTrace();
        }
    }

    public Usuario getUser(){

        Usuario u = null;

        String selectQuery = "SELECT  * FROM " + TABLE_USER;
        SQLiteDatabase db = this.getWritableDatabase();

        try{

            Cursor cursor = db.rawQuery(selectQuery, null);

            if (cursor.moveToFirst()) {

                u = new Usuario();
                u.setId(cursor.getInt(0));
                u.setNome(cursor.getString(1));
                u.setEmail(cursor.getString(2));
                u.setDtNasc(cursor.getString(3));
                u.setSenha(cursor.getString(4));
                u.setImagem(cursor.getString(5));
                u.setSobre(cursor.getString(6));

                String estilos     = cursor.getString(7);
                String influencias = cursor.getString(8);

                List<String> ls = new ArrayList<>();
                String[] estilos_array = estilos.split(",");
                for ( String s : estilos_array ){
                    if ( !s.isEmpty() && !s.equals("null") )
                        ls.add( s  );
                }
                u.setEstilos(ls);

                ls = new ArrayList<>();
                String[] influ = influencias.split(",");
                for ( String s : influ ) {
                    if ( !s.isEmpty() && !s.equals("null") )
                        ls.add( s  );
                }
                u.setInfluencias(ls);

                u.setLatitude(cursor.getDouble(9));
                u.setLongitude(cursor.getDouble(10));
                u.setCidade(cursor.getString(11));
                u.setEstado(cursor.getString(12));
                u.setPais(cursor.getString(13));

                db.close();

                u.setInstrumentos(getInstrumentos(u));
            }

        }
        catch (Exception e){
            e.printStackTrace();
        }

        return u;
    }

    public void updateUser(Usuario u){

        SQLiteDatabase db = this.getWritableDatabase();
        ContentValues values = new ContentValues();

        try {

            String estilos = "", influencias = "";

            if ( u.getEstilos() != null && !u.getEstilos().isEmpty() ){
                for ( String s : u.getEstilos() ){
                    estilos = estilos + s + ",";
                }
            }

            if ( u.getInfluencias() != null && !u.getInfluencias().isEmpty() ){
                for ( String s : u.getInfluencias() ){
                    influencias = influencias + s + ",";
                }
            }

            values.put(KEY_NAME,   u.getNome());       // User Name
            values.put(KEY_EMAIL,  u.getEmail());      // User Email
            values.put(KEY_DATE,   u.getDtNasc());     // User Birth
            values.put(KEY_PASS,   u.getSenha());      // User Password
            values.put(KEY_IMG,    u.getImagem());     // User Image
            values.put(KEY_ABOUT,  u.getSobre());      // User About
            values.put(KEY_STYLES, estilos);           // User Styles
            values.put(KEY_INFL,   influencias);       // User Influences
            values.put(KEY_LAT,    u.getLatitude());   // User Lat
            values.put(KEY_LONG,   u.getLongitude());  // User Long
            values.put(KEY_CITY,   u.getCidade());     // User City
            values.put(KEY_STATE,  u.getEstado());     // User State
            values.put(KEY_PAIS,   u.getPais());       // User Country

            db.update(TABLE_USER, values, KEY_ID + " = ? ", new String[] { String.valueOf(u.getId()) });

        }
        catch ( Exception e ){
            e.printStackTrace();
        }
        finally {
            db.close();
        }

    }

    public void deleteUser(Usuario u){

        SQLiteDatabase db = this.getWritableDatabase();

        try{

            db.delete(TABLE_USER, KEY_ID + " = ?",
                    new String[]{String.valueOf(u.getId())});

            db.execSQL("DELETE FROM " + TABLE_FOLLOW);
            db.execSQL("DELETE FROM " + TABLE_MESSAGES);
            db.execSQL("DELETE FROM " + TABLE_LMUPDATE);

        }
        catch ( Exception e ){
            e.printStackTrace();
        }
        finally {
            db.close();
        }
    }

    public void addFriend(Usuario u){

        SQLiteDatabase db = this.getWritableDatabase();
        ContentValues values = new ContentValues();

        try {

            values.put(KEY_ID, u.getId());       // User ID
            values.put(KEY_NAME, u.getNome());   // User Name
            values.put(KEY_IMG, u.getImagem());  // User Image
            values.put(KEY_CITY, u.getCidade()); // User City

            db.insert(TABLE_FOLLOW, null, values);

        }
        catch ( Exception e ){
            e.printStackTrace();
        }
        finally {
            db.close();
        }

    }

    public List<Usuario> getFriends(){

        List<Usuario> lFriends = new ArrayList<>();
        SQLiteDatabase db = this.getWritableDatabase();

        try {

            String selectQuery = "SELECT  * FROM " + TABLE_FOLLOW;
            Cursor cursor = db.rawQuery(selectQuery, null);

            if (cursor.moveToFirst()) {

                do{

                    Usuario u = new Usuario();
                    u.setId(cursor.getInt(0));
                    u.setNome(cursor.getString(1));
                    u.setImagem(cursor.getString(2));
                    u.setCidade(cursor.getString(3));

                    lFriends.add(u);

                }while ( cursor.moveToNext() );

            }
        }
        catch ( Exception e ){
            e.printStackTrace();
        }
        finally {
            db.close();
        }

        return lFriends;
    }

    public void deleteFriend(Usuario u){

        SQLiteDatabase db = this.getWritableDatabase();

        try{
            db.delete(TABLE_FOLLOW, KEY_ID + " = ?",
                    new String[] { String.valueOf(u.getId()) });
        }
        catch ( Exception e ){
            e.printStackTrace();
        }
        finally {
            db.close();
        }

    }

    public boolean isFriend(Usuario u){

        SQLiteDatabase db = this.getWritableDatabase();

        String query = "SELECT * FROM " + TABLE_FOLLOW
                + " WHERE " + KEY_ID + " = " + u.getId();

        Cursor cursor = db.rawQuery(query, null);

        if ( cursor.moveToFirst() ){

            db.close();
            return true;
        }
        else{

            db.close();
            return false;

        }
    }

    public Usuario getFriend(int id){

        Usuario u = new Usuario();
        SQLiteDatabase db = this.getWritableDatabase();

        try{

            String query = "SELECT * FROM " + TABLE_FOLLOW
                    + " WHERE " + KEY_ID + " = " + id;

            Cursor cursor = db.rawQuery(query, null);

            if ( cursor.moveToFirst() ){

                u.setId(cursor.getInt(0));
                u.setNome(cursor.getString(1));
                u.setImagem(cursor.getString(2));
                u.setCidade(cursor.getString(3));

            }

        }
        catch (Exception e){
            e.printStackTrace();
        }
        finally {
            db.close();
        }

        return u;
    }

    public void addInstrument(Usuario u){

        if ( u.getInstrumentos() != null && !u.getInstrumentos().isEmpty() ) {

            SQLiteDatabase db = this.getWritableDatabase();
            ContentValues values = new ContentValues();

            try {

                db.execSQL("DELETE FROM " + TABLE_UINST);

                for (Instrumento i : u.getInstrumentos()) {

                    values.put(KEY_ID, i.getInstId());       // Inst ID
                    values.put(KEY_TYPE, i.getInstType());   // Inst Type
                    values.put(KEY_NAME, i.getInstName());   // Inst Name
                    values.put(KEY_LEVEL, i.getInstLevel()); // Inst Level

                    db.insert(TABLE_UINST, null, values);
                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                db.close();
            }

        }
    }

    public List<Instrumento> getInstrumentos(Usuario u){

        List<Instrumento> linst = null;

        String selectQuery = "SELECT  * FROM " + TABLE_UINST;
        SQLiteDatabase db = this.getWritableDatabase();

        try{

            Cursor cursor = db.rawQuery(selectQuery, null);

            if (cursor.moveToFirst()) {

                linst = new ArrayList<>();

                do {

                    Instrumento i = new Instrumento();
                    i.setInstId(cursor.getInt(0));
                    i.setInstType(cursor.getInt(1));
                    i.setInstName(cursor.getString(2));
                    i.setInstLevel(cursor.getInt(3));

                    linst.add(i);

                }while ( cursor.moveToNext() );
            }
        }
        catch (Exception e){
            e.printStackTrace();
        }
        finally {
            db.close();
        }

        return linst;
    }

    public void setLastUpdate(int id, String date){

        SQLiteDatabase db = this.getWritableDatabase();
        ContentValues values = new ContentValues();

        try {

            values.put(KEY_ID, id);
            values.put(KEY_DATE, date);

            db.execSQL("DELETE FROM " + TABLE_LMUPDATE + " WHERE id = " + id);

            db.insert(TABLE_LMUPDATE, null, values);
            db.close();

        }
        catch ( Exception e ){
            e.printStackTrace();
        }
    }

    public String getLastUpdate(int id){

        String date = "";

        SQLiteDatabase db = this.getWritableDatabase();

        try {

            String selectQuery = "SELECT * FROM " + TABLE_LMUPDATE
                    + " WHERE " + KEY_ID + " = " + id;

            Cursor cursor = db.rawQuery(selectQuery, null);

            if ( cursor.moveToFirst() )
                date = cursor.getString(1);

            db.close();
        }
        catch ( Exception e ){
            e.printStackTrace();
        }

        return date;
    }

    public void addMessage(Message message) {

        SQLiteDatabase db = this.getWritableDatabase();
        ContentValues values = new ContentValues();

        try{

            values.put("chatID", message.getChatId());
            values.put("userID", message.getUsuario().getId());
            values.put("msgID", message.getMsgId());
            values.put("texto", message.getTexto());
            values.put("date", message.getMsgDate());

            if ( message.getVisto() ){
                values.put("visto", "X");
            }
            else
                values.put("visto", "");

            db.insert(TABLE_MESSAGES, null, values);

        }
        catch ( Exception e ){
            e.printStackTrace();
        }
        finally {
            db.close();
        }
    }

    public List<Message> getChatList(int chatID){

        List<Message> lmessages = new ArrayList<>();
        SQLiteDatabase db = this.getWritableDatabase();

        try {

            String selectQuery = "SELECT * FROM " + TABLE_MESSAGES + " WHERE chatID = " + chatID +
                    " ORDER BY msgID ASC;" ;

            Cursor cursor = db.rawQuery(selectQuery, null);
            if ( cursor.moveToFirst() ){

                do {

                    Message message = new Message();
                    Usuario user = new Usuario();
                    user.setId(cursor.getInt(1));

                    message.setChatId(cursor.getInt(0));
                    message.setUsuario(user);
                    message.setMsgId(cursor.getInt(2));
                    message.setTexto(cursor.getString(3));
                    message.setMsgDate(cursor.getString(4));

                    lmessages.add(message);

                }while ( cursor.moveToNext() );
            }
        }
        catch (Exception erro){
            erro.printStackTrace();
        }

        return lmessages;
    }

    public int getCountUnreadMessages(){

        SQLiteDatabase db = this.getWritableDatabase();

        try {

            String selectQuery = "SELECT * FROM " + TABLE_MESSAGES + " WHERE visto != 'X';";
            Cursor cursor = db.rawQuery(selectQuery, null);

            if ( cursor.moveToFirst() )
                return cursor.getCount();

        }
        catch (Exception erro){
            erro.printStackTrace();
        }

        return 0;
    }

    public Usuario getUserMessage(int id){

        SQLiteDatabase db = this.getWritableDatabase();

        try {

            String query = "SELECT * FROM USERS"
                    + " WHERE " + KEY_ID + " = " + id;

            Cursor cursor = db.rawQuery(query, null);

            if ( cursor.moveToFirst() ){

                Usuario u = new Usuario();

                u.setId(cursor.getInt(0));
                u.setNome(cursor.getString(1));
                u.setImagem(cursor.getString(2));
                u.setCidade(cursor.getString(3));

                return u;

            }
        }
        catch (Exception e){
            e.printStackTrace();
        }
        finally {
            db.close();
        }

        return null;
    }

    public void setUserMessage(Usuario u) {

        SQLiteDatabase db = this.getWritableDatabase();
        ContentValues values = new ContentValues();

        try {

            values.put(KEY_ID, u.getId());       // User ID
            values.put(KEY_NAME, u.getNome());   // User Name
            values.put(KEY_IMG, u.getImagem());  // User Image
            values.put(KEY_CITY, u.getCidade()); // User City

            db.insert("USERS", null, values);

        }
        catch ( Exception e ){
            e.printStackTrace();
        }
        finally {
            db.close();
        }
    }

    public String getLastChatMessage(int chatId) {

        SQLiteDatabase db = this.getWritableDatabase();

        try {

            String selectQuery = "SELECT chatID, texto FROM " + TABLE_MESSAGES
                    + " WHERE chatID = " + chatId + " ORDER BY msgID DESC" ;
            Cursor cursor = db.rawQuery(selectQuery, null);
            if ( cursor.moveToFirst() ){

                return cursor.getString(1);

            }
        }
        catch (Exception erro){
            erro.printStackTrace();
        }


        return null;
    }

    public void setUserChat(int chatId, int id) {

        SQLiteDatabase db = this.getWritableDatabase();

        try {

            String selectQuery = "SELECT * FROM CHATUSER WHERE chatId = " + chatId
                    + " AND id = " + id + "; ";

            Cursor cursor = db.rawQuery(selectQuery, null);
            if ( !cursor.moveToFirst() ){

                ContentValues values = new ContentValues();
                values.put("chatId", chatId);
                values.put("id", id);

                db.insert("CHATUSER", null, values);
            }

        }
        catch ( Exception e ){
            e.printStackTrace();
        }
        finally {
            db.close();
        }
    }

    public int getUserChat(int id){

        SQLiteDatabase db = this.getWritableDatabase();

        try{

            String selectQuery = "SELECT chatId FROM CHATUSER WHERE id = " + id + ";";
            Cursor cursor = db.rawQuery(selectQuery, null);
            if ( cursor.moveToFirst() ) {
                return cursor.getInt(0);
            }
        }
        catch (Exception erro){
            erro.printStackTrace();
        }
        return 0;
    }
}
