package Util;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.DialogFragment;
import android.content.DialogInterface;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.audiofx.Visualizer;
import android.os.Bundle;
import android.os.Environment;
import android.support.v4.content.ContextCompat;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageButton;
import android.widget.SeekBar;
import android.widget.TextView;

import java.io.File;
import java.io.IOException;

import butterknife.ButterKnife;
import butterknife.InjectView;
import tads.ufpr.pr.br.backstage.R;

public class PlayDialog extends DialogFragment {

    @InjectView(R.id.btnStop)       ImageButton     _btnStop;
    @InjectView(R.id.btnPlay)       ImageButton     _btnPlay;
    @InjectView(R.id.btnPause)      ImageButton     _btnPause;
    @InjectView(R.id.visualizer)    VisualizerView  _visualizer;

    private String file;
    Visualizer mVisualizer;
    private String filePath;
    private int id;
    MediaPlayer mPlayer = null;

    boolean isPaused, isPlaying;

    public void setFileName(String file, int id){
        this.id = id;
        this.file = file;
    }

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        AlertDialog.Builder b = new AlertDialog.Builder(getActivity())
                .setTitle(file)
                .setIcon(ContextCompat.getDrawable(getActivity(), R.drawable.records_icon2))
                .setNegativeButton("Fechar",
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int whichButton) {

                                if ( mPlayer != null ){

                                    mPlayer.stop();
                                    mPlayer.reset();
                                    mPlayer.release();
                                    mPlayer = null;

                                }

                                dialog.cancel();
                                dialog.dismiss();
                            }
                        }
                );

        getActivity().setVolumeControlStream(AudioManager.STREAM_MUSIC);

        LayoutInflater i = getActivity().getLayoutInflater();
        View rootView = i.inflate(R.layout.player_layout, null);

        ButterKnife.inject(this, rootView);

        _btnStop.setEnabled(false);
        _btnStop.setAlpha((float) 0.3);
        _btnPause.setEnabled(false);
        _btnPause.setAlpha((float) 0.3);

        _btnPlay.setOnClickListener(clickEvent);
        _btnPause.setOnClickListener(clickEvent);
        _btnStop.setOnClickListener(clickEvent);

        return b.setView(rootView).create();
    }

    @Override
    public void onStart() {
        super.onStart();

        if (getDialog() == null) {
            return;
        }
        getDialog().setCanceledOnTouchOutside(false);
        int titleDividerId = getDialog().getContext().getResources().getIdentifier("alertTitle", "id", "android");
        View titleDivider = getDialog().findViewById(titleDividerId);
        if (titleDivider != null){
            ((TextView)titleDivider).setTextColor(ContextCompat.getColor(getActivity(), R.color.color_primary_dark));
        }
    }


    View.OnClickListener clickEvent = new View.OnClickListener() {

        @Override
        public void onClick(View v) {

            switch ( v.getId() ){

                case R.id.btnPlay:

                    playAction();
                    break;

                case R.id.btnPause:

                    pauseAction();
                    break;

                case R.id.btnStop:

                    stopAction();
                    break;
            }
        }
    };

    private void stopAction() {

        mVisualizer.setEnabled(false);

        mPlayer.stop();
        mPlayer.reset();
        mPlayer.release();
        mPlayer = null;

        isPaused = false;
        isPlaying = false;

        _btnPause.setAlpha((float) 0.3);
        _btnPause.setEnabled(false);
        _btnPlay.setAlpha((float) 1);
        _btnPlay.setEnabled(true);
        _btnStop.setAlpha((float) 0.3);
        _btnStop.setEnabled(false);
    }

    private void pauseAction() {

        mPlayer.pause();

        isPaused = true;
        isPlaying = false;

        _btnPause.setAlpha((float) 0.3);
        _btnPause.setEnabled(false);
        _btnPlay.setAlpha((float) 1);
        _btnPlay.setEnabled(true);
    }

    private void playAction() {

        if ( isPaused ){

            isPlaying = true;
            isPaused  = false;

            mPlayer.start();

            _btnPlay.setAlpha((float) 0.3);
            _btnPlay.setEnabled(false);
            _btnPause.setAlpha((float) 1);
            _btnPause.setEnabled(true);

        }
        else {

            if ( id != 0 ){

                File path = new File( Environment.getExternalStorageDirectory().getAbsolutePath()
                        + getString(R.string.filepath) + id + "/" + file );
                filePath = path.getPath();

            }
            else {

                File path = new File( Environment.getExternalStorageDirectory().getAbsolutePath()
                        + getString(R.string.filepath) + file );
                filePath = path.getPath();

            }

            try {

                _btnPlay.setAlpha((float) 0.3);
                _btnPlay.setEnabled(false);
                _btnPause.setAlpha((float) 1);
                _btnPause.setEnabled(true);
                _btnStop.setAlpha((float) 1);
                _btnStop.setEnabled(true);

                if ( mPlayer == null )
                    mPlayer = new MediaPlayer();

                mPlayer.setOnCompletionListener(new MediaPlayer.OnCompletionListener() {

                    @Override
                    public void onCompletion(MediaPlayer mp) {

                        mVisualizer.setEnabled(false);

                        mPlayer.stop();
                        mPlayer.reset();
                        mPlayer.release();
                        mPlayer = null;

                        _btnStop.setAlpha((float) 0.3);
                        _btnStop.setEnabled(false);
                        _btnPlay.setAlpha((float) 1);
                        _btnPlay.setEnabled(true);
                        _btnPause.setAlpha((float) 0.3);
                        _btnPause.setEnabled(false);

                        isPlaying = false;
                        isPaused = false;

                    }
                });

                setupVisualizerPlayer();
                mVisualizer.setEnabled(true);

                mPlayer.setDataSource(filePath);
                mPlayer.prepare();
                mPlayer.start();

                isPlaying = true;

            } catch (IOException e) {
                e.printStackTrace();
            }

        }
    }

    private void setupVisualizerPlayer() {

        mVisualizer = new Visualizer(mPlayer.getAudioSessionId());
        mVisualizer.setCaptureSize(Visualizer.getCaptureSizeRange()[1]);
        mVisualizer.setDataCaptureListener(
                new Visualizer.OnDataCaptureListener() {
                    public void onWaveFormDataCapture(Visualizer visualizer,
                                                      byte[] bytes, int samplingRate) {
                        _visualizer.updateVisualizer(bytes);
                    }

                    public void onFftDataCapture(Visualizer visualizer,
                                                 byte[] bytes, int samplingRate) {
                    }
                }, Visualizer.getMaxCaptureRate() / 2, true, false);
    }
}
