package Util;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.DialogFragment;
import android.content.DialogInterface;
import android.graphics.Typeface;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.MediaRecorder;
import android.media.audiofx.Visualizer;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.SystemClock;
import android.support.v4.content.ContextCompat;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Chronometer;
import android.widget.ImageButton;
import android.widget.TextView;
import java.io.File;
import java.io.IOException;
import butterknife.ButterKnife;
import butterknife.InjectView;
import tads.ufpr.pr.br.backstage.AppController;
import tads.ufpr.pr.br.backstage.R;
import tads.ufpr.pr.br.backstage.Usuario;

public class RecordDialog extends DialogFragment {

    @InjectView(R.id.progress_clock) Chronometer    _pgClock;
    @InjectView(R.id.btnStop)        ImageButton    _btnStop;
    @InjectView(R.id.btnPlayPause)   ImageButton    _btnPlayPause;
    @InjectView(R.id.btnRec)         ImageButton    _btnRec;
    @InjectView(R.id.visualizer)     VisualizerView _visualizer;

    Usuario u;
    String fpath;
    boolean isPaused;
    boolean isPlaying;
    boolean isRecording;
    Visualizer mVisualizer;
    MediaPlayer mPlayer = null;
    MediaRecorder mRecorder = null;

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        LayoutInflater i = getActivity().getLayoutInflater();
        View rootView = i.inflate(R.layout.record_layout, null);

        getActivity().setVolumeControlStream(AudioManager.STREAM_MUSIC);
        u = ((AppController)getActivity().getApplication()).getUsuario();

        AlertDialog.Builder b = new AlertDialog.Builder(getActivity())
                .setTitle("Gravação de Áudio")
                .setIcon(ContextCompat.getDrawable(getActivity(), R.drawable.record2))
                /*
                .setPositiveButton("Publicar",
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int whichButton) {

                            }
                        }
                )
                */
                .setNegativeButton("Fechar",
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int whichButton) {

                                if ( isPlaying ){

                                    mPlayer.stop();
                                    mPlayer.reset();
                                    mPlayer.release();
                                    mPlayer = null;

                                }
                                else if ( isRecording ){

                                    mRecorder.stop();
                                    mRecorder.release();
                                    mRecorder = null;

                                }


                                dialog.cancel();
                                dialog.dismiss();
                            }
                        }
                );


        ButterKnife.inject(this, rootView);

        Typeface tf  = Typeface.createFromAsset(getActivity().getAssets(), "GeosansLight.ttf");
        _pgClock.setTypeface(tf);

        _btnStop.setAlpha((float) 0.3);
        _btnPlayPause.setAlpha((float) 0.3);
        _pgClock.setAlpha((float) 0.3);

        _btnStop.setEnabled(false);
        _btnPlayPause.setEnabled(false);

        _btnStop.setOnClickListener(clickEvent);
        _btnPlayPause.setOnClickListener(clickEvent);
        _btnRec.setOnClickListener(clickEvent);

        return b.setView(rootView).create();

    }

    @Override
    public void onStart() {
        super.onStart();

        if (getDialog() == null) {
            return;
        }
        getDialog().setCanceledOnTouchOutside(false);
        int titleDividerId = getDialog().getContext().getResources().getIdentifier("alertTitle", "id", "android");
        View titleDivider = getDialog().findViewById(titleDividerId);
        if (titleDivider != null){
            ((TextView)titleDivider).setTextColor(ContextCompat.getColor(getActivity(), R.color.color_primary_dark));
        }
    }

    View.OnClickListener clickEvent = new View.OnClickListener() {

        @Override
        public void onClick(View v) {

            switch ( v.getId() ){

                case R.id.btnRec:

                    startRecording();
                    break;

                case R.id.btnStop:

                    stopAction();
                    break;

                case R.id.btnPlayPause:

                    playPauseAction();
                    break;
            }
        }
    };

    @Override
    public void onPause() {
        super.onPause();

        if ( getActivity().isFinishing() && mPlayer != null) {
            mVisualizer.release();
            _visualizer.setEnabled(false);
            mPlayer.release();
            mPlayer = null;
        }
    }

    private void startRecording() {

        _btnStop.setAlpha((float) 1.0);
        _pgClock.setAlpha((float) 1.0);
        _btnRec.setAlpha((float) 0.3);
        _btnRec.setEnabled(false);
        _btnStop.setEnabled(true);

        mPlayer = new MediaPlayer();

        try {

            File path = new File( Environment.getExternalStorageDirectory().getAbsolutePath()
                    + getString(R.string.filepath) + "/" + u.getId() + "/");
            path.mkdirs();
            path = File.createTempFile("temp_", ".mp3", path);

            fpath = Environment.getExternalStorageDirectory().getAbsolutePath()
                    + getString(R.string.filepath) + "/" + u.getId() + "/" + path.getName();

            path.delete();

            _btnPlayPause.setAlpha((float) .3);
            _btnPlayPause.setEnabled(false);

            mRecorder = new MediaRecorder();
            mRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);
            mRecorder.setOutputFormat(MediaRecorder.OutputFormat.MPEG_4);
            mRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AAC);
            mRecorder.setAudioEncodingBitRate(96000);
            mRecorder.setAudioSamplingRate(44100);
            mRecorder.setOutputFile(fpath);
            mRecorder.prepare();

            _pgClock.setBase(SystemClock.elapsedRealtime());
            _pgClock.start();

            mRecorder.start();
            isRecording = true;

        }
        catch (IOException e) {
            e.printStackTrace();
        }

        _pgClock.start();

    }

    private void stopAction() {

        _btnStop.setAlpha((float) 0.3);
        _btnStop.setEnabled(false);

        if ( isRecording) {

            _pgClock.stop();
            _pgClock.setAlpha((float) 0.3);

            _btnRec.setAlpha((float) 1.0);
            _btnRec.setEnabled(true);

            _btnPlayPause.setAlpha((float) 1.0);
            _btnPlayPause.setEnabled(true);

            mRecorder.stop();
            mRecorder.release();
            mRecorder = null;

            isRecording = false;

        }
        else {

            isPlaying = false;
            isPaused  = false;

            if ( mVisualizer != null )
                mVisualizer.setEnabled(false);

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN)
                _btnPlayPause.setBackground(ContextCompat.getDrawable(getActivity(), R.drawable.play));
            else
                _btnPlayPause.setBackgroundDrawable(ContextCompat.getDrawable(getActivity(), R.drawable.play));

            mPlayer.stop();
            mPlayer.reset();
            mPlayer.release();
            mPlayer = null;

            _btnPlayPause.setEnabled(true);
            _btnPlayPause.setAlpha((float) 1.0);

            _btnRec.setAlpha((float) 1.0);
            _btnRec.setEnabled(true);

        }
    }

    private void playPauseAction() {

        if ( !isPlaying && !isPaused ){

            isPlaying = true;
            isPaused = false;

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN)
                _btnPlayPause.setBackground(ContextCompat.getDrawable(getActivity(), R.drawable.pause));
            else
                _btnPlayPause.setBackgroundDrawable(ContextCompat.getDrawable(getActivity(), R.drawable.pause));

            _btnRec.setAlpha((float) 0.3);
            _btnRec.setEnabled(false);

            _btnStop.setAlpha((float) 1.0);
            _btnStop.setEnabled(true);

            try {

                if ( mPlayer == null )
                    mPlayer = new MediaPlayer();

                mPlayer.setOnCompletionListener(new MediaPlayer.OnCompletionListener() {

                    @Override
                    public void onCompletion(MediaPlayer mp) {

                        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN)
                            _btnPlayPause.setBackground(ContextCompat.getDrawable(getActivity(), R.drawable.play));
                        else
                            _btnPlayPause.setBackgroundDrawable(ContextCompat.getDrawable(getActivity(), R.drawable.play));

                        mVisualizer.setEnabled(false);

                        mPlayer.stop();
                        mPlayer.reset();
                        mPlayer.release();
                        mPlayer = null;

                        _btnRec.setAlpha((float) 1.0);
                        _btnRec.setEnabled(true);

                        _btnStop.setAlpha((float) 0.3);
                        _btnStop.setEnabled(false);

                        isPlaying = false;
                        isPaused = false;
                    }
                });

                setupVisualizerPlayer();
                mVisualizer.setEnabled(true);

                mPlayer.setDataSource(fpath);
                mPlayer.prepare();
                mPlayer.start();

            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        else if ( isPlaying ){

            isPlaying = false;
            isPaused = true;

            mPlayer.pause();

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN)
                _btnPlayPause.setBackground(ContextCompat.getDrawable(getActivity(), R.drawable.play));
            else
                _btnPlayPause.setBackgroundDrawable(ContextCompat.getDrawable(getActivity(), R.drawable.play));

        }
        else{

            _btnStop.setAlpha((float) 1.0);
            _btnStop.setEnabled(true);

            mPlayer.start();
            isPlaying = true;
            isPaused  = false;

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN)
                _btnPlayPause.setBackground(ContextCompat.getDrawable(getActivity(), R.drawable.pause));
            else
                _btnPlayPause.setBackgroundDrawable(ContextCompat.getDrawable(getActivity(), R.drawable.pause));

        }
    }

    private void setupVisualizerPlayer() {

        mVisualizer = new Visualizer(mPlayer.getAudioSessionId());
        mVisualizer.setCaptureSize(Visualizer.getCaptureSizeRange()[1]);
        mVisualizer.setDataCaptureListener(
                new Visualizer.OnDataCaptureListener() {
                    public void onWaveFormDataCapture(Visualizer visualizer,
                                                      byte[] bytes, int samplingRate) {
                        _visualizer.updateVisualizer(bytes);
                    }

                    public void onFftDataCapture(Visualizer visualizer,
                                                 byte[] bytes, int samplingRate) {
                    }
                }, Visualizer.getMaxCaptureRate() / 2, true, false);
    }

}
