package Util;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Environment;
import android.provider.MediaStore;
import android.util.Base64;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MIME;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.HTTP;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.security.KeyStore;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Formatter;

import tads.ufpr.pr.br.backstage.R;
import tads.ufpr.pr.br.backstage.Usuario;

import static org.apache.http.params.HttpConnectionParams.setConnectionTimeout;
import static org.apache.http.params.HttpConnectionParams.setSoTimeout;

@SuppressWarnings("All")
public class UploadFile extends AsyncTask<Void, Integer, String> {

    private Uri picUri;
    private String texto;
    private String url;
    HttpResponse response;
    private Usuario usuario;
    private String audioName;
    private Activity activity;
    ProgressDialog progressDialog;

    //Post Text
    public UploadFile(Activity activity, Usuario usuario,  String texto){
        this.activity = activity;
        this.usuario  = usuario;
        this.texto    = texto;
    }

    //Upload Image file
    public UploadFile(Activity activity, Usuario usuario, Uri picUri, String texto){
        this.activity = activity;
        this.picUri   = picUri;
        this.usuario  = usuario;
        this.texto    = texto;
    }

    //Upload Audio file
    public UploadFile(Activity activity, Usuario usuario, String audioName, String texto){
        this.activity  = activity;
        this.audioName = audioName;
        this.usuario   = usuario;
        this.texto     = texto;
    }

    //Post Link
    public UploadFile(Activity activity, Usuario usuario, Object o, String url, String texto) {
        this.activity  = activity;
        this.usuario   = usuario;
        this.texto     = texto;
        this.url       = url;
    }

    @Override
    protected void onPreExecute() {
        super.onPreExecute();

        progressDialog = new ProgressDialog(activity, R.style.AppTheme_Dark_Dialog);
        progressDialog.setIndeterminate(true);
        progressDialog.setCancelable(false);
        progressDialog.setMessage("Realizando postagem...");
        progressDialog.show();

    }

    @Override
    protected void onPostExecute(String s) {
        super.onPostExecute(s);

        progressDialog.cancel();
        progressDialog.dismiss();

        String ret = "";

        int statusCode = response.getStatusLine().getStatusCode();

        if ( statusCode == 201 ){

            AlertDialog.Builder adb = new AlertDialog.Builder(activity, R.style.AppTheme_Dark_Dialog);
            adb.setTitle("Yeah!!!");
            adb.setMessage("Seu post foi realizado com sucesso! Agora me dê um 'High Five' e volte para a tela principal...");

            adb.setPositiveButton("Hi5", new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int id) {
                    dialog.cancel();
                    activity.finish();
                }
            });
            AlertDialog ad = adb.create();
            ad.show();
        }
    }

    @Override
    protected String doInBackground(Void... params) {

        try{

            KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
            trustStore.load(null, null);

            MySSLSocketFactory sf = new MySSLSocketFactory(trustStore);
            sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);

            BasicHttpParams httpParams = new BasicHttpParams();
            setConnectionTimeout(httpParams, 3000);
            setSoTimeout(httpParams, 5000);

            HttpProtocolParams.setVersion(httpParams, HttpVersion.HTTP_1_1);
            HttpProtocolParams.setContentCharset(httpParams, HTTP.UTF_8);

            SchemeRegistry registry = new SchemeRegistry();
            registry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
            registry.register(new Scheme("https", sf, 443));

            ClientConnectionManager ccm = new ThreadSafeClientConnManager(httpParams, registry);

            HttpClient httpClient = new DefaultHttpClient(ccm, httpParams);

            httpClient = new DefaultHttpClient(httpParams);

            HttpPost httpPost = new HttpPost( activity.getString(R.string.server_address)
                    + "/usuario/" + usuario.getId() + "/postagem" );

            String user = activity.getResources().getString(R.string.server_user);
            String basicAuth = "Basic " + Base64.encodeToString(user.getBytes(), Base64.NO_WRAP);
            httpPost.setHeader("Authorization", basicAuth);

            MultipartEntityBuilder entityBuilder = MultipartEntityBuilder.create();
            entityBuilder.setMode(HttpMultipartMode.BROWSER_COMPATIBLE);
            entityBuilder.setCharset(Charset.forName("UTF-8"));

            if ( picUri != null ){

                String[] proj = { MediaStore.Audio.Media.DATA };
                Cursor cursor = activity.managedQuery(picUri, proj, null, null, null);
                int column_index = cursor.getColumnIndexOrThrow(MediaStore.Audio.Media.DATA);
                cursor.moveToFirst();

                File sourceFile = new File(cursor.getString(column_index));

                entityBuilder.addBinaryBody("file", sourceFile, ContentType.MULTIPART_FORM_DATA, getShaString(sourceFile));
                entityBuilder.addTextBody("type", "3");
                entityBuilder.addTextBody("text", texto , ContentType.create("text/plain", MIME.UTF8_CHARSET) );

            }
            else if ( audioName != null ){

                File sourceFile = new File(Environment.getExternalStorageDirectory().getAbsolutePath()
                        + activity.getString(R.string.filepath) + "/" + usuario.getId() + "/" + audioName);

                entityBuilder.addBinaryBody("file", sourceFile, ContentType.APPLICATION_OCTET_STREAM, getShaString(sourceFile));
                entityBuilder.addTextBody("type", "4");
                entityBuilder.addTextBody("text", texto, ContentType.create("text/plain", MIME.UTF8_CHARSET));

            }
            else if ( url != null ){

                entityBuilder.addTextBody("type", "2");
                entityBuilder.addTextBody("text", texto , ContentType.create("text/plain", MIME.UTF8_CHARSET));
                entityBuilder.addTextBody("link", url);

            }
            else {

                entityBuilder.addTextBody("type", "1");
                entityBuilder.addTextBody("text", texto , ContentType.create("text/plain", MIME.UTF8_CHARSET));
                entityBuilder.addTextBody("link", "");

            }

            HttpEntity entity = entityBuilder.build();
            httpPost.setEntity(entity);

            response = httpClient.execute(httpPost);

            httpPost.abort();
            httpClient.getConnectionManager().closeExpiredConnections();
            httpClient.getConnectionManager().shutdown();

        }
        catch(Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    private String getShaString(File sourceFile) {

        MessageDigest md = null;
        String sha = "";

        try {

            md = MessageDigest.getInstance("SHA1");

            InputStream in = new FileInputStream(sourceFile);

            byte[] buf = new byte[8192];
            for (;;) {
                int len = in.read(buf);
                if (len < 0)
                    break;
                md.update(buf, 0, len);
            }
            in.close();

            byte[] hash = md.digest();

            Formatter formatter = new Formatter();
            for (byte b : hash) {
                formatter.format("%02x", b);
            }

            sha = formatter.toString();

        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return sha;

    }

}
