package Util;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.res.TypedArray;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.util.Base64;
import android.util.Log;

import java.io.ByteArrayOutputStream;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Calendar;
import tads.ufpr.pr.br.backstage.R;

public class Utils {

    public static boolean isNetworkOnline(Context context) {

        try{
            ConnectivityManager cm = (ConnectivityManager)context.getSystemService(Context.CONNECTIVITY_SERVICE);
            if ( cm != null ){
                NetworkInfo activeNetworkInfo = cm.getActiveNetworkInfo();
                return activeNetworkInfo != null && activeNetworkInfo.isConnected();
            }
            else
                return false;
        }catch(Exception e){
            e.printStackTrace();
            return false;
        }
    }

    public static void alertDialog(Context context, String title, String text, int opt){

        AlertDialog.Builder adb = new AlertDialog.Builder(context, R.style.AppTheme_Dark_Dialog);
        adb.setTitle(title);
        adb.setMessage(text);

        if(opt == 1) {
            adb.setPositiveButton("Ok", new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog,int id) {
                    dialog.cancel();
                }
            });
            AlertDialog ad = adb.create();
            ad.show();
        }
    }

    public static int getToolbarHeight(Context context) {
        final TypedArray styledAttributes = context.getTheme().obtainStyledAttributes(
                new int[]{R.attr.actionBarSize});
        int toolbarHeight = (int) styledAttributes.getDimension(0, 0);
        styledAttributes.recycle();
        return toolbarHeight;
    }

    public static int getTabsHeight(Context context) {
        return (int) context.getResources().getDimension(R.dimen.tabsHeight);
    }

    public static Bitmap string2Bitmap(String s) {
        try {

            byte[] decodedByte = Base64.decode(s, 0);
            return BitmapFactory.decodeByteArray(decodedByte, 0, decodedByte.length);

        }
        catch ( Exception e ){
            Log.e("Erro", e.getMessage());
            e.printStackTrace();
        }

        return null;
    }

    public static String bitmap2String(Bitmap image){
        Bitmap immagex = image;
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        immagex.compress(Bitmap.CompressFormat.JPEG, 90, baos);
        byte[] b = baos.toByteArray();
        return Base64.encodeToString(b, Base64.DEFAULT);
    }

    public static int calcAge(String data){

        int dia = Integer.parseInt(data.substring(8, 10));
        int mes = Integer.parseInt(data.substring(5, 7));
        int ano = Integer.parseInt(data.substring(0, 4));

        Calendar nasc = Calendar.getInstance();
        Calendar hoje = Calendar.getInstance();

        nasc.set(ano, mes, dia);

        int idade = hoje.get(Calendar.YEAR) - nasc.get(Calendar.YEAR);
        if (hoje.get(Calendar.DAY_OF_YEAR) < nasc.get(Calendar.DAY_OF_YEAR)){
            idade--;
        }

        return idade;
    }

    public static String getMd5Hash(String input) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] messageDigest = md.digest(input.getBytes());
            BigInteger number = new BigInteger(1, messageDigest);
            String md5 = number.toString(16);

            while (md5.length() < 32)
                md5 = "0" + md5;

            return md5;
        } catch (NoSuchAlgorithmException e) {
            Log.e("MD5", e.getLocalizedMessage());
            return null;
        }
    }
}
