package tads.ufpr.pr.br.backstage;

import android.app.Activity;
import android.app.Application;
import android.text.TextUtils;
import android.util.Base64;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.toolbox.ImageLoader;
import com.android.volley.toolbox.Volley;

import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import java.net.URLDecoder;
import java.security.KeyStore;
import java.util.ArrayList;
import java.util.List;

import Util.MySSLSocketFactory;
import volley.LruBitmapCache;

import static org.apache.http.params.HttpConnectionParams.setConnectionTimeout;
import static org.apache.http.params.HttpConnectionParams.setSoTimeout;

@SuppressWarnings("All")
public class AppController extends Application {

    public static final String TAG = AppController.class.getSimpleName();

    private RequestQueue mRequestQueue;
    private ImageLoader mImageLoader;
    LruBitmapCache mLruBitmapCache;
    HttpResponse response;
    private Usuario usuario;
    private boolean action, updated, updatedError;
    private List<InstType> listInstType;

    private static AppController mInstance;

    @Override
    public void onCreate() {
        super.onCreate();
        mInstance = this;
    }

    public static synchronized AppController getInstance() {
        return mInstance;
    }

    public RequestQueue getRequestQueue() {
        if (mRequestQueue == null) {
            mRequestQueue = Volley.newRequestQueue(getApplicationContext());
        }

        return mRequestQueue;
    }

    public ImageLoader getImageLoader() {
        getRequestQueue();
        if (mImageLoader == null) {
            getLruBitmapCache();
            mImageLoader = new ImageLoader(this.mRequestQueue, mLruBitmapCache);
        }

        return this.mImageLoader;
    }

    public LruBitmapCache getLruBitmapCache() {
        if (mLruBitmapCache == null)
            mLruBitmapCache = new LruBitmapCache();
        return this.mLruBitmapCache;
    }

    public <T> void addToRequestQueue(Request<T> req, String tag) {
        req.setTag(TextUtils.isEmpty(tag) ? TAG : tag);
        getRequestQueue().add(req);
    }

    public <T> void addToRequestQueue(Request<T> req) {
        req.setTag(TAG);
        getRequestQueue().add(req);
    }

    public void cancelPendingRequests(Object tag) {
        if (mRequestQueue != null) {
            mRequestQueue.cancelAll(tag);
        }
    }

    public Usuario getUsuario() {
        return usuario;
    }

    public void setUsuario(Usuario usuario) {
        this.usuario = usuario;
    }

    public List<InstType> getListInstType() {
        return listInstType;
    }

    public void setListInstType(List<InstType> listInstType) {
        this.listInstType = listInstType;
    }

    public void instantiateInstList(Activity activity) {

        new Thread() {

            @Override
            public void run() {

                try{

                    KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
                    trustStore.load(null, null);

                    MySSLSocketFactory sf = new MySSLSocketFactory(trustStore);
                    sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);

                    BasicHttpParams httpParams = new BasicHttpParams();
                    setConnectionTimeout(httpParams, 3000);
                    setSoTimeout(httpParams, 5000);

                    HttpProtocolParams.setVersion(httpParams, HttpVersion.HTTP_1_1);
                    HttpProtocolParams.setContentCharset(httpParams, HTTP.UTF_8);

                    SchemeRegistry registry = new SchemeRegistry();
                    registry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
                    registry.register(new Scheme("https", sf, 443));

                    ClientConnectionManager ccm = new ThreadSafeClientConnManager(httpParams, registry);

                    List<InstType> lType = new ArrayList<>();
                    String[] TypesArray = getResources().getStringArray(R.array.types);
                    int index_type = 1;
                    for ( String s : TypesArray ) {

                        InstType it = new InstType();
                        it.setTypeId(index_type);
                        it.setTypeName(s);

                        switch (index_type) {

                            case 1: //Cordas

                                it.setTypeLogo(R.drawable.chords);
                                break;

                            case 2: //Percussão

                                it.setTypeLogo(R.drawable.drums);
                                break;

                            case 3: //Sopro

                                it.setTypeLogo(R.drawable.sax);
                                break;

                            case 4: //Teclas

                                it.setTypeLogo(R.drawable.teclado);
                                break;

                            case 5: //Vocal

                                it.setTypeLogo(R.drawable.mic);
                                break;
                        }

                        lType.add(it);
                        index_type++;

                    }

                    String url = getResources().getString(R.string.server_address) + "/instrumento";
                    HttpGet httpGet = new HttpGet(url);

                    String user = getResources().getString(R.string.server_user);
                    String basicAuth = "Basic " + Base64.encodeToString(user.getBytes(), Base64.NO_WRAP);
                    httpGet.setHeader("Authorization", basicAuth);

                    HttpClient httpclient = new DefaultHttpClient(ccm, httpParams);
                    //HttpClient httpclient = new DefaultHttpClient(httpParams);

                    response = httpclient.execute(httpGet);
                    String result = URLDecoder.decode(EntityUtils.toString(response.getEntity()), HTTP.UTF_8);

                    if ( !result.isEmpty() ){

                        JSONArray ja = new JSONArray(result);

                        List<Instrumento> lChords = new ArrayList<>();
                        List<Instrumento> lDrums  = new ArrayList<>();
                        List<Instrumento> lWind   = new ArrayList<>();
                        List<Instrumento> lKeys   = new ArrayList<>();
                        List<Instrumento> lVoice  = new ArrayList<>();

                        if ( ja != null ){

                            for ( int i = 0 ; i < ja.length() ; i++ ){

                                JSONObject jo = ja.getJSONObject(i);
                                Instrumento instrumento = new Instrumento();
                                instrumento.setInstId(jo.getInt("id"));
                                instrumento.setInstName(new String(jo.getString("nome").getBytes("ISO-8859-1"), HTTP.UTF_8));

                                switch (jo.getInt("tipo_instrumento_id")) {

                                    case 1: //Cordas

                                        lChords.add(instrumento);
                                        break;

                                    case 2: //Percussão

                                        lDrums.add(instrumento);
                                        break;

                                    case 3: //Sopro

                                        lWind.add(instrumento);
                                        break;

                                    case 4: //Teclas

                                        lKeys.add(instrumento);
                                        break;

                                    case 5: //Vocal

                                        lVoice.add(instrumento);
                                        break;
                                }
                            }

                            lType.get(0).setInstrumentos(lChords);
                            lType.get(1).setInstrumentos(lDrums);
                            lType.get(2).setInstrumentos(lWind);
                            lType.get(3).setInstrumentos(lKeys);
                            lType.get(4).setInstrumentos(lVoice);

                            setListInstType(lType);
                        }
                    }

                    response.getEntity().consumeContent();
                    httpGet.abort();
                    httpclient.getConnectionManager().closeExpiredConnections();
                    httpclient.getConnectionManager().shutdown();

                }
                catch ( Exception error ){
                    error.printStackTrace();
                }
            }
        }.start();
    }


    public boolean isAction() {
        return action;
    }

    public void setAction(boolean action) {
        this.action = action;
    }

    public boolean isUpdated() {
        return updated;
    }

    public void setUpdated(boolean updated) {
        this.updated = updated;
    }

    public boolean isUpdatedError() {
        return updatedError;
    }

    public void setUpdatedError(boolean updatedError) {
        this.updatedError = updatedError;
    }
}
