package tads.ufpr.pr.br.backstage;

import android.app.Activity;
import android.graphics.Bitmap;
import android.os.AsyncTask;
import android.os.Handler;
import android.support.design.widget.FloatingActionButton;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.Toolbar;
import android.util.Base64;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONObject;

import java.net.URLDecoder;
import java.security.KeyStore;
import java.util.ArrayList;
import java.util.List;

import Util.MySSLSocketFactory;
import Util.RoundImage;
import Util.Utils;
import butterknife.ButterKnife;
import butterknife.InjectView;

import static org.apache.http.params.HttpConnectionParams.setConnectionTimeout;
import static org.apache.http.params.HttpConnectionParams.setSoTimeout;

@SuppressWarnings("All")
public class ChatActivity extends AppCompatActivity {

    @InjectView(R.id.btnSend)   FloatingActionButton    _btnSend;
    @InjectView(R.id.inputText) EditText                _etInput;

    int chatID;
    Usuario me;
    Usuario person;
    Handler handler;
    ListView _lvMessages;
    List<Message> listMessage;
    MessagesListAdapter _mlAdapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_chat);

        ButterKnife.inject(this);

        me = ((AppController)this.getApplication()).getUsuario();

        _lvMessages = (ListView)findViewById(R.id.list_view_messages);

        Bundle extras = getIntent().getExtras();
        if (extras != null)
            person = (Usuario) extras.getSerializable("person");

        initToolbar();

        listMessage = new ArrayList<>();

        _mlAdapter = new MessagesListAdapter(this, listMessage, me, person);
        _lvMessages.setAdapter(_mlAdapter);

        _btnSend.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {

                if (!_etInput.getText().toString().isEmpty()) {

                    String text = _etInput.getText().toString();
                    _etInput.setText("");

                    if (Utils.isNetworkOnline(ChatActivity.this)) {

                        new SendMessage(me.getId(), person.getId(), text,
                                ChatActivity.this).execute();

                    } else {

                        Toast.makeText(ChatActivity.this, "Ops! Erro ao enviar mensagem." +
                                " Tente novamente mais tarde", Toast.LENGTH_SHORT);

                    }
                }
            }
        });

        handler = new Handler();
        handler.postDelayed(new Runnable() {
            public void run() {

                if (Utils.isNetworkOnline(ChatActivity.this))
                    new ReadMessages(me.getId(), person.getId()).execute();

                if (handler != null)
                    handler.postDelayed(this, 1000);

            }
        }, 1000);

    }

    @Override
    protected void onStop() {
        super.onStop();
        handler = null;
    }

    private void initToolbar() {

        final Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        ((TextView)findViewById(R.id.usrName)).setText(person.getNome());

        if ( person.getImagem() != null && !person.getImagem().isEmpty()
                && !person.getImagem().equals("null") ){

            Bitmap bm = Utils.string2Bitmap(person.getImagem());
            RoundImage rm = new RoundImage(bm);
            ((ImageView)findViewById(R.id.usrIcon)).setImageDrawable(rm);

        }
        else{

            ((ImageView)findViewById(R.id.usrIcon)).setImageResource(R.drawable.aboutme);

        }

        (findViewById(R.id.usrIcon)).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Toast.makeText(ChatActivity.this, "Voltar", Toast.LENGTH_SHORT).show();
            }
        });
    }

    private class ReadMessages extends AsyncTask<Void, Void, Void> {

        private int me, user;
        List<Message> listMessages2;
        private HttpClient httpClient;

        public ReadMessages(int me, int user){
            this.me = me;
            this.user = user;
        }

        @Override
        protected void onPreExecute() {
            super.onPreExecute();

            listMessages2 = new ArrayList<>();

        }

        @Override
        protected Void doInBackground(Void... params) {

            try {

                BasicHttpParams httpParams = new BasicHttpParams();
                setConnectionTimeout(httpParams, 3000);
                setSoTimeout(httpParams, 5000);

                HttpProtocolParams.setVersion(httpParams, HttpVersion.HTTP_1_1);
                HttpProtocolParams.setContentCharset(httpParams, HTTP.UTF_8);

                httpClient = new DefaultHttpClient(httpParams);

                while( chatID == 0 ) {
                    getCharID();
                }

                String url = getString(R.string.server_address) + "/usuario/" + me
                        + "/chat/" + chatID + "/mensagem" ;

                HttpGet httpGet = new HttpGet(url);

                String suser = getApplicationContext().getString(R.string.server_user);
                String basicAuth = "Basic " + Base64.encodeToString(suser.getBytes(), Base64.NO_WRAP);
                httpGet.setHeader("Authorization", basicAuth);

                HttpResponse httpResponse = httpClient.execute(httpGet);
                if ( httpResponse != null ){

                    int statCode = httpResponse.getStatusLine().getStatusCode();
                    String ret = URLDecoder.decode(EntityUtils.toString(httpResponse.getEntity()),
                            HTTP.UTF_8);

                    if ( statCode == 200 ){

                        JSONArray ja = new JSONArray(ret);

                        if ( ja.length() > 0 )
                            for ( int i = 0 ; i < ja.length() ; i++ ){

                                JSONObject jo = ja.getJSONObject(i);
                                Message m = new Message();
                                m.setChatId(jo.getInt("chat_id"));
                                m.setMsgId(jo.getInt("id"));
                                m.setTexto(new String(jo.getString("texto")
                                        .getBytes("ISO-8859-1"), HTTP.UTF_8));

                                Usuario u = new Usuario();
                                u.setId(jo.getInt("usuario_id"));
                                m.setUsuario(u);

                                listMessages2.add(m);

                            }
                    }
                }

                if ( httpGet != null )
                    httpGet.abort();

                httpClient.getConnectionManager().closeExpiredConnections();
                httpClient.getConnectionManager().shutdown();

            }
            catch (Exception e) { e.printStackTrace(); }


            return null;
        }

        @Override
        protected void onPostExecute(Void aVoid) {
            super.onPostExecute(aVoid);

            if ( listMessages2.size() > 0 && listMessage.size() > 0 ){

                if ( listMessages2.get(0).getMsgId() > listMessage.get(0).getMsgId() ){

                    listMessage.clear();
                    listMessage.addAll(listMessages2);

                    for ( int i = listMessages2.size() ; i > 0 ; i-- ){

                        listMessage.add( listMessages2.get( i - 1 ) );

                    }

                    _mlAdapter.notifyDataSetChanged();

                }

            }
            else if ( listMessages2.size() > 0 ){

                listMessage.clear();
                listMessage.addAll(listMessages2);

                for ( int i = listMessages2.size() ; i > 0 ; i-- ){

                    listMessage.add( listMessages2.get( i - 1 ) );

                }

                _mlAdapter.notifyDataSetChanged();

            }

        }

        private void getCharID() {

            try {

                //Início criação do Chat
                String url = getString(R.string.server_address) + "/usuario/" + me
                        + "/chat";

                List<NameValuePair> lp = new ArrayList<>();
                lp.add(new BasicNameValuePair("name", "Chat do usuario " + user));
                lp.add(new BasicNameValuePair("uid", String.valueOf(user)));

                UrlEncodedFormEntity urle = new UrlEncodedFormEntity(lp, HTTP.UTF_8);

                HttpPost httpPost = new HttpPost(url);
                httpPost.setEntity(urle);

                String suser = getString(R.string.server_user);
                String basicAuth = "Basic " + Base64.encodeToString(suser.getBytes(), Base64.NO_WRAP);
                httpPost.setHeader("Authorization", basicAuth);

                HttpResponse httpResponse = httpClient.execute(httpPost);

                if ( httpResponse != null) {

                    int statCode = httpResponse.getStatusLine().getStatusCode();
                    String ret = URLDecoder.decode(EntityUtils.toString(httpResponse.getEntity()),
                            HTTP.UTF_8);

                    if (statCode == 200) {

                        JSONArray ja = new JSONArray(ret);
                        JSONObject jo = ja.getJSONObject(0);

                        chatID = jo.getInt("new_chat");

                    }

                    httpResponse.getEntity().consumeContent();

                }

                if ( httpPost != null )
                    httpPost.abort();

            }
            catch (Exception e) { e.printStackTrace(); }
        }

    }

    private class SendMessage extends AsyncTask<Void, Void, Void> {

        private int me, user;
        private String message;
        private Activity activity;
        private HttpClient httpClient;

        public SendMessage (int me, int user, String message, Activity activity){

            this.me = me;
            this.user = user;
            this.message = message;
            this.activity = activity;

            Message m = new Message();
            m.setTexto(message);
            Usuario ur = new Usuario();
            ur.setId(me);
            m.setUsuario(ur);

            if ( listMessage == null )
                listMessage = new ArrayList<>();

            listMessage.add(m);
            _mlAdapter.notifyDataSetChanged();

        }

        @Override
        protected Void doInBackground(Void... params) {

            try {

                KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
                trustStore.load(null, null);

                MySSLSocketFactory sf = new MySSLSocketFactory(trustStore);
                sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);

                BasicHttpParams httpParams = new BasicHttpParams();
                setConnectionTimeout(httpParams, 3000);
                setSoTimeout(httpParams, 5000);

                HttpProtocolParams.setVersion(httpParams, HttpVersion.HTTP_1_1);
                HttpProtocolParams.setContentCharset(httpParams, HTTP.UTF_8);

                SchemeRegistry registry = new SchemeRegistry();
                registry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
                registry.register(new Scheme("https", sf, 443));

                ClientConnectionManager ccm = new ThreadSafeClientConnManager(httpParams, registry);

                HttpClient httpclient = new DefaultHttpClient(ccm, httpParams);

                httpClient = new DefaultHttpClient(httpParams);

                while( chatID == 0 ) {
                    getCharID();
                }

                sendMessage(me, chatID, activity, message);

            }
            catch (Exception e) { e.printStackTrace(); }

            return null;
        }

        private void getCharID() {

            try {

                //Início criação do Chat
                String url = getString(R.string.server_address) + "/usuario/" + me
                        + "/chat";

                List<NameValuePair> lp = new ArrayList<>();
                lp.add(new BasicNameValuePair("name", "Chat do usuario " + user));
                lp.add(new BasicNameValuePair("uid", String.valueOf(user)));

                UrlEncodedFormEntity urle = new UrlEncodedFormEntity(lp, HTTP.UTF_8);

                HttpPost httpPost = new HttpPost(url);
                httpPost.setEntity(urle);

                String suser = getString(R.string.server_user);
                String basicAuth = "Basic " + Base64.encodeToString(suser.getBytes(), Base64.NO_WRAP);
                httpPost.setHeader("Authorization", basicAuth);

                HttpResponse httpResponse = httpClient.execute(httpPost);

                if ( httpResponse != null) {

                    int statCode = httpResponse.getStatusLine().getStatusCode();
                    String ret = URLDecoder.decode(EntityUtils.toString(httpResponse.getEntity()),
                            HTTP.UTF_8);

                    if (statCode == 200) {

                        JSONArray ja = new JSONArray(ret);
                        JSONObject jo = ja.getJSONObject(0);

                        chatID = jo.getInt("new_chat");

                    }

                    httpResponse.getEntity().consumeContent();

                }

                if ( httpPost != null )
                    httpPost.abort();

                httpClient.getConnectionManager().closeExpiredConnections();
                httpClient.getConnectionManager().shutdown();

            }
            catch (Exception e) { e.printStackTrace(); }
        }

        private void sendMessage(int me, int chatId, Activity activity, String message) {

            try {

                KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
                trustStore.load(null, null);

                MySSLSocketFactory sf = new MySSLSocketFactory(trustStore);
                sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);

                BasicHttpParams httpParams = new BasicHttpParams();
                setConnectionTimeout(httpParams, 3000);
                setSoTimeout(httpParams, 5000);

                HttpProtocolParams.setVersion(httpParams, HttpVersion.HTTP_1_1);
                HttpProtocolParams.setContentCharset(httpParams, HTTP.UTF_8);

                SchemeRegistry registry = new SchemeRegistry();
                registry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
                registry.register(new Scheme("https", sf, 443));

                ClientConnectionManager ccm = new ThreadSafeClientConnManager(httpParams, registry);
                httpClient = new DefaultHttpClient(ccm, httpParams);

                //httpClient = new DefaultHttpClient(httpParams);

                String url = activity.getString(R.string.server_address) + "/usuario/" + me
                        + "/chat/" + chatId +"/mensagem";

                List<NameValuePair> lp = new ArrayList<>();
                lp.add(new BasicNameValuePair("text", message));

                UrlEncodedFormEntity urle = new UrlEncodedFormEntity(lp, HTTP.UTF_8);
                HttpPost httpPost = new HttpPost(url);
                httpPost.setEntity(urle);

                String suser = activity.getString(R.string.server_user);
                String basicAuth = "Basic " + Base64.encodeToString(suser.getBytes(), Base64.NO_WRAP);
                httpPost.setHeader("Authorization", basicAuth);

                HttpResponse httpResponse = httpClient.execute(httpPost);
                if ( httpResponse != null ){

                    int statCode = httpResponse.getStatusLine().getStatusCode();
                    String ret = URLDecoder.decode(EntityUtils.toString(httpResponse.getEntity()),
                            HTTP.UTF_8);

                }
                else
                    Toast.makeText(ChatActivity.this, "Ops! Erro ao enviar mensagem." +
                            " Tente novamente mais tarde", Toast.LENGTH_SHORT);

            }
            catch (Exception e) { e.printStackTrace(); }
        }
    }
}
