package tads.ufpr.pr.br.backstage;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.app.FragmentManager;
import android.app.FragmentTransaction;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Parcelable;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

public class EditActivity extends AppCompatActivity {

    private static final int INSTRUMENTOS = 300;
    private static final int PERFIL       = 400;
    private static final int PREFERENCIAS = 500;

    private Usuario u;

    AlertDialog.Builder adbuilder;
    Toolbar toolbar;
    Button btnCanc;
    Button btnSave;
    int action;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_edit);

        Bundle extras = getIntent().getExtras();
        u = (Usuario) extras.getSerializable("user");

        toolbar = (Toolbar)findViewById(R.id.toolbar);
        toolbar.setTitleTextColor(ContextCompat.getColor(this, R.color.white));
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        adbuilder = new AlertDialog.Builder(EditActivity.this);

        btnCanc = (Button)findViewById(R.id.btnCanc);
        btnSave = (Button)findViewById(R.id.btnSave);

        FragmentManager fragmentManager = getFragmentManager();
        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();

        View.OnTouchListener touchListener = new View.OnTouchListener(){

            @SuppressLint("ClickableViewAccessibility")
            public boolean onTouch(View v, MotionEvent event) {
                switch (event.getAction()) {
                    case MotionEvent.ACTION_DOWN: {
                        v.setBackgroundColor(0x35000000);
                        v.invalidate();
                        break;
                    }
                    case MotionEvent.ACTION_UP: {
                        v.setBackgroundColor(0x00000000);
                        v.invalidate();
                        break;
                    }
                }
                return false;
            }
        };

        btnCanc.setOnTouchListener(touchListener);
        btnSave.setOnTouchListener(touchListener);

        action = extras.getInt("action");

        if ( action == PERFIL ){
            getSupportActionBar().setTitle(R.string.edit_profile);
            EditProfileFragment editProfile = new EditProfileFragment();
            fragmentTransaction.add(R.id.frag_edit_container, editProfile, "EDIT_PROFILE");
        }
        else if ( action == INSTRUMENTOS ){
            getSupportActionBar().setTitle(R.string.edit_instruments);
            EditInstrumentosFragment editInstruments = new EditInstrumentosFragment();
            fragmentTransaction.add(R.id.frag_edit_container, editInstruments, "EDIT_INSTRUMENTS");
        }
        else {
            getSupportActionBar().setTitle(getString(R.string.edit_preferences) + " - " + extras.getString("preference"));
            EditPreferenciasFragment editPreferences = new EditPreferenciasFragment();
            fragmentTransaction.add(R.id.frag_edit_container, editPreferences, "EDIT_PREFERENCES");
        }

        fragmentTransaction.commit();

        btnSave.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {

                InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.hideSoftInputFromWindow(v.getWindowToken(), 0);

                adbuilder.setTitle(getString(R.string.save_changes_title));
                adbuilder.setMessage(getString(R.string.save_changes_text));

                adbuilder.setNegativeButton(getString(R.string.cancel), new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        dialog.cancel();
                    }
                });

                adbuilder.setPositiveButton(getString(R.string.save), new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        dialog.cancel();

                        if ( action == PERFIL ) {

                            String nome  = ((EditText) EditActivity.this.findViewById(R.id.input_name)).getText().toString();
                            String sobre = ((EditText) EditActivity.this.findViewById(R.id.input_about_me)).getText().toString();
                            String date  = ((EditText) EditActivity.this.findViewById(R.id.input_date)).getText().toString();

                            try {

                                DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
                                df.setLenient(false);
                                Date dt = df.parse(date);
                                df = new SimpleDateFormat("yyyy-MM-dd");

                                u.setDtNasc(df.format(dt).toString());
                                u.setNome(nome);
                                u.setSobre(sobre);

                                Usuario usr = ((AppController)getApplication()).getUsuario();

                                u.setCidade(usr.getCidade());
                                u.setEstado(usr.getEstado());
                                u.setPais(usr.getPais());

                                ((AppController) getApplication()).setUsuario(u);
                                setResult(RESULT_OK);
                                finish();

                            }
                            catch (ParseException e) {
                                e.printStackTrace();
                            }
                        }
                        else if ( action == INSTRUMENTOS || action == PREFERENCIAS ){
                            setResult(RESULT_OK);
                            finish();
                        }

                    }
                });
                adbuilder.show();
            }
        });

        btnCanc.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                cancChanges();
            }
        });
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {

        if ( action == INSTRUMENTOS )
            getMenuInflater().inflate(R.menu.menu_instrumentos, menu);

        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        if ( item.getItemId() != R.id.menu_add_instrument )
            cancChanges();

        return false;
    }

    @Override
    public void onBackPressed() {
        cancChanges();
    }

    private void cancChanges() {

        adbuilder.setTitle(getString(R.string.canc_changes_title));
        adbuilder.setMessage(getString(R.string.canc_changes_text));

        adbuilder.setNegativeButton(getString(R.string.keep_editing),
                new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                dialog.cancel();
            }
        });

        adbuilder.setPositiveButton(getString(R.string.cancel),
                new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                dialog.cancel();
                setResult(RESULT_CANCELED);
                EditActivity.this.finish();
            }
        });
        adbuilder.show();
    }

    @Override
    protected void onSaveInstanceState(Bundle outState){
        super.onSaveInstanceState(outState);
        outState.putSerializable("usuario", u);
        outState.putParcelableArrayList("instList", (ArrayList<? extends Parcelable>)
                ((AppController) this.getApplication()).getListInstType());
    }

    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        u = (Usuario)savedInstanceState.getSerializable("usuario");
        ((AppController)this.getApplication()).setUsuario(u);
        ((AppController)this.getApplication()).setListInstType(
                savedInstanceState.<InstType>getParcelableArrayList("instList"));
    }
}
