package tads.ufpr.pr.br.backstage;

import android.app.Fragment;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.location.Location;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.provider.Settings;
import android.support.design.widget.Snackbar;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import java.util.Calendar;

import Util.AppLocationService;
import Util.LocationAddress;
import butterknife.ButterKnife;
import butterknife.InjectView;

public class EditProfileFragment extends Fragment implements AdapterView.OnItemSelectedListener {

    @InjectView(R.id.input_name)            EditText _inputNome;
    @InjectView(R.id.input_date)            EditText _inputDate;
    @InjectView(R.id.input_about_me)        EditText _inputSobre;
    @InjectView(R.id.btn_update_location)   Button   _btnUpdate;

    View v;
    Usuario u;
    ProgressDialog progressDialog;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        v = inflater.inflate(R.layout.edit_profile_fragment, container, false);

        u = ((AppController)getActivity().getApplication()).getUsuario();
        ButterKnife.inject(this, v);

        _inputNome.setText(u.getNome());

        if ( u.getSobre() != null && !u.getSobre().isEmpty()) {
            _inputSobre.setText(u.getSobre());
        }

        if ( u.getDtNasc() != null && !u.getDtNasc().isEmpty() ){

            String date = u.getDtNasc().substring(8, 10) + "/" + u.getDtNasc().substring(5, 7)
                    + "/" + u.getDtNasc().substring(0, 4);

            _inputDate.setText(date);
        }

        setDateListener();

        setLocation();

        _btnUpdate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                updateLocation();
            }
        });

        return v;
    }

    private void setDateListener() {

        _inputDate.addTextChangedListener(new TextWatcher() {

            private String current = "";
            private String ddmmyyyy = "DDMMYYYY";
            private Calendar cal = Calendar.getInstance();
            private Calendar cal2 = Calendar.getInstance();

            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                //Do Nothing
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

                if (!s.toString().equals(current)) {

                    String clean = s.toString().replaceAll("[^\\d.]", "");
                    String cleanC = current.replaceAll("[^\\d.]", "");

                    int cl = clean.length();
                    int sel = cl;
                    for (int i = 2; i <= cl && i < 6; i += 2) {
                        sel++;
                    }

                    if (clean.equals(cleanC)) sel--;

                    if (clean.length() < 8)
                        clean = clean + ddmmyyyy.substring(clean.length());

                    else {
                        int day = Integer.parseInt(clean.substring(0, 2));
                        int mon = Integer.parseInt(clean.substring(2, 4));
                        int year = Integer.parseInt(clean.substring(4, 8));

                        if (mon > 12) mon = 12;
                        cal.set(Calendar.MONTH, mon - 1);
                        int minYear = cal2.get(Calendar.YEAR) - 80;
                        int maxYear = cal2.get(Calendar.YEAR) - 6;
                        year = (year < minYear) ? minYear : (year > maxYear) ? maxYear : year;
                        cal.set(Calendar.YEAR, year);

                        day = (day > cal.getActualMaximum(Calendar.DATE)) ? cal.getActualMaximum(Calendar.DATE) : day;
                        clean = String.format("%02d%02d%02d", day, mon, year);
                    }

                    clean = String.format("%s/%s/%s", clean.substring(0, 2),
                            clean.substring(2, 4),
                            clean.substring(4, 8));

                    sel = sel < 0 ? 0 : sel;
                    current = clean;
                    _inputDate.setText(current);
                    _inputDate.setSelection(sel < current.length() ? sel : current.length());
                }

            }

            @Override
            public void afterTextChanged(Editable s) {
                //Do Nothing
            }
        });
    }

    private void setLocation() {

        if ( u.getCidade() != null && !u.getCidade().isEmpty())
            ((TextView) v.findViewById(R.id.tv_cityName)).setText(u.getCidade());
        else
            ((TextView) v.findViewById(R.id.tv_cityName)).setText(getString(R.string.city));

        if (u.getEstado() != null && !u.getEstado().isEmpty() && u.getPais() != null && !u.getPais().isEmpty())
            ((TextView) v.findViewById(R.id.tv_stateCountry)).setText(u.getEstado() + ", " + u.getPais());
        else if (u.getEstado() != null && !u.getEstado().isEmpty())
            ((TextView) v.findViewById(R.id.tv_stateCountry)).setText(u.getEstado() + ", " + getString(R.string.country));
        else if (u.getPais() != null && !u.getPais().isEmpty())
            ((TextView) v.findViewById(R.id.tv_stateCountry)).setText(getString(R.string.state) + ", " + u.getPais());
        else
            ((TextView) v.findViewById(R.id.tv_stateCountry)).setText(getString(R.string.state) + ", " + getString(R.string.country));

    }

    public void updateLocation(){

        AppLocationService appLocationService = new AppLocationService(getActivity());

        if ( appLocationService.isLocationEnabled() ) {

            progressDialog = new ProgressDialog(getActivity(), R.style.AppTheme_Dark_Dialog);
            progressDialog.setIndeterminate(true);
            progressDialog.setCancelable(false);
            progressDialog.setMessage(getResources().getString(R.string.getting_location));
            progressDialog.show();

            Location location = null;

            while ( location == null ) {
                location = appLocationService.getLocation(getActivity(), LocationManager.GPS_PROVIDER, LocationManager.NETWORK_PROVIDER);
            }

            double latitude =  location.getLatitude();
            double longitude = location.getLongitude();

            u.setLatitude(latitude);
            u.setLongitude(longitude);

            LocationAddress locationAddress = new LocationAddress();
            LocationAddress.getAddressFromLocation(latitude, longitude, getActivity().getApplicationContext(), new GeocoderHandler());

        }
        else
            showSettingsAlert();
    }

    private class GeocoderHandler extends Handler {

        @Override
        public void handleMessage(Message message) {

            switch (message.what) {

                case 10:

                    progressDialog.cancel();
                    progressDialog.dismiss();

                    Bundle bundle = message.getData();

                    u.setCidade(bundle.getString("city"));
                    u.setEstado(bundle.getString("state"));
                    u.setPais(bundle.getString("country"));

                    ((AppController)getActivity().getApplication()).setUsuario(u);

                    setLocation();

                    Snackbar.make(getActivity().findViewById(R.id.coordinatorLayout),
                            "Localização atualizada!",
                            Snackbar.LENGTH_LONG).show();

                    break;

                case 11:

                    Snackbar.make(getActivity().findViewById(R.id.coordinatorLayout),
                            "Não foi possível recuperar a localizacão...",
                            Snackbar.LENGTH_LONG).show();

                    break;
            }
        }
    }

    public void showSettingsAlert() {

        android.support.v7.app.AlertDialog.Builder alertDialog = new android.support.v7.app.AlertDialog.Builder( getActivity() );

        alertDialog.setTitle(getString(R.string.settings));
        alertDialog.setMessage(getString(R.string.enable_location));
        alertDialog.setCancelable(false);

        alertDialog.setPositiveButton(getString(R.string.settings), new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                dialog.cancel();
                Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                getActivity().startActivity(intent);
            }
        });

        alertDialog.setNegativeButton(getString(R.string.cancel), new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                dialog.cancel();
            }
        });

        alertDialog.show();
    }

    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
        //Do Nothing
    }

    @Override
    public void onNothingSelected(AdapterView<?> parent) {
        //Do Nothing
    }
}