package tads.ufpr.pr.br.backstage;

import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.support.design.widget.TabLayout;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentStatePagerAdapter;
import android.support.v4.content.ContextCompat;
import android.support.v4.view.ViewPager;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.style.ImageSpan;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.animation.AccelerateInterpolator;
import android.view.animation.Animation;
import android.view.animation.DecelerateInterpolator;
import android.view.animation.ScaleAnimation;

import com.getbase.floatingactionbutton.FloatingActionButton;
import com.getbase.floatingactionbutton.FloatingActionsMenu;
import java.util.ArrayList;
import java.util.List;
import Util.DatabaseHandler;
import Util.RecordDialog;
import Util.Utils;
import butterknife.ButterKnife;
import butterknife.InjectView;

public class MainActivity extends AppCompatActivity {

    @InjectView(R.id.fabButton)   FloatingActionsMenu  _menu;
    @InjectView(R.id.new_post)    FloatingActionButton _newPost;
    @InjectView(R.id.new_record)  FloatingActionButton _newRecord;

    private int[] imageResId = {
        R.drawable.friends_icon,
        R.drawable.feed_icon,
        R.drawable.records_icon
    };

    Usuario u;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        u = ((AppController)this.getApplication()).getUsuario();

        ButterKnife.inject(this);

        initToolbar();
        initViewPagerAndTabs();

        _newRecord.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                _menu.toggle();
                android.app.FragmentManager fm = MainActivity.this.getFragmentManager();
                RecordDialog dialogFragment = new RecordDialog();
                dialogFragment.show(fm, "Record Fragment");
            }
        });

        _newPost.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                _menu.toggle();
                Intent i = new Intent(MainActivity.this, PostActivity.class);
                startActivity(i);
            }
        });
    }

    private void initViewPagerAndTabs() {

        final ViewPager viewPager = (ViewPager) findViewById(R.id.viewPager);
        ViewPagerAdapter adapter =  new ViewPagerAdapter(getSupportFragmentManager(), 3);
        viewPager.setAdapter(adapter);

        TabLayout tabLayout = (TabLayout) findViewById(R.id.tabLayout);
        tabLayout.setupWithViewPager(viewPager);

        tabLayout.getTabAt(0).setText(getPageTitle(0));
        tabLayout.getTabAt(1).setText(getPageTitle(1));
        tabLayout.getTabAt(2).setText(getPageTitle(2));

        viewPager.setCurrentItem(1);

        tabLayout.setOnTabSelectedListener(new TabLayout.OnTabSelectedListener() {

            @Override
            public void onTabSelected(TabLayout.Tab tab) {

                viewPager.setCurrentItem(tab.getPosition());

                if (tab.getPosition() == 0 || tab.getPosition() == 2) {
                    hideButton();
                } else {
                    showButton();
                }
            }

            @Override
            public void onTabUnselected(TabLayout.Tab tab) {

            }

            @Override
            public void onTabReselected(TabLayout.Tab tab) {

            }
        });

    }

    private void showButton() {

        _menu.clearAnimation();

        ScaleAnimation expand =  new ScaleAnimation(0f, 1f, 0f, 1f, Animation.RELATIVE_TO_SELF, 1f, Animation.RELATIVE_TO_SELF, 1f);
        expand.setDuration(200);
        expand.setInterpolator(new DecelerateInterpolator());

        expand.setAnimationListener(new Animation.AnimationListener() {
            @Override
            public void onAnimationStart(Animation animation) {

            }

            @Override
            public void onAnimationEnd(Animation animation) {
                _menu.setVisibility(View.VISIBLE);
            }

            @Override
            public void onAnimationRepeat(Animation animation) {

            }
        });
        _menu.startAnimation(expand);

    }

    private void hideButton() {

        if ( _menu.getVisibility() == View.VISIBLE ){

            _menu.clearAnimation();

            ScaleAnimation shrink =  new ScaleAnimation(1f, 0f, 1f, 0f, Animation.RELATIVE_TO_SELF, 1f, Animation.RELATIVE_TO_SELF, 1f);
            shrink.setDuration(200);
            shrink.setInterpolator(new AccelerateInterpolator());
            shrink.setAnimationListener(new Animation.AnimationListener() {

                @Override
                public void onAnimationStart(Animation animation) { }

                @Override
                public void onAnimationEnd(Animation animation) {
                    _menu.setVisibility(View.GONE);
                }

                @Override
                public void onAnimationRepeat(Animation animation) { }

            });

            _menu.startAnimation(shrink);

        }

    }

    private void initToolbar() {
        Toolbar mToolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);
        setTitle(getString(R.string.app_name));
        mToolbar.setTitleTextColor(ContextCompat.getColor(this, R.color.white));
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    protected void onSaveInstanceState(Bundle outState){
        super.onSaveInstanceState(outState);
        outState.putSerializable("usuario", u);
    }

    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);

        u = (Usuario)savedInstanceState.getSerializable("usuario");
        ((AppController)this.getApplication()).setUsuario(u);

        if ( ((AppController) this.getApplication()).getListInstType() == null ||
                ((AppController) this.getApplication()).getListInstType().isEmpty() )
            ((AppController) this.getApplication()).instantiateInstList(this);

        if ( u == null ){

            DatabaseHandler dbh = new DatabaseHandler(this);
            dbh.deleteUser(u);
            Intent i = new Intent(this, LoginActivity.class);
            startActivity(i);
            finish();

        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int id = item.getItemId();

        switch (id){

            case R.id.menu_perfil:

                Intent i = new Intent(this, ProfileActivity.class);
                startActivity(i);
                break;

            case R.id.menu_search:

                i = new Intent(this, SearchActivity.class);
                startActivity(i);
                break;

            case R.id.menu_msgs:

                i = new Intent(this, MessagesActivity.class);
                startActivity(i);
                break;

            case R.id.menu_leave:

                DatabaseHandler dbh = new DatabaseHandler(this);
                dbh.deleteUser(u);
                i = new Intent(this, LoginActivity.class);
                startActivity(i);
                finish();

                break;

        }
        return super.onOptionsItemSelected(item);
    }

    public CharSequence getPageTitle(int tab) {
        Drawable image = ContextCompat.getDrawable(this, imageResId[tab]);

        int tb = Utils.getTabsHeight(this);
        tb -= tb * 0.5;

        image.setBounds(0, 0, tb , tb );
        SpannableString sb = new SpannableString(" ");
        ImageSpan imageSpan = new ImageSpan(image, ImageSpan.ALIGN_BOTTOM);
        sb.setSpan(imageSpan, 0, 1, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
        return sb;
    }

    public class ViewPagerAdapter extends FragmentStatePagerAdapter {

        CharSequence Titles[];
        int NumbOfTabs;

        public ViewPagerAdapter(FragmentManager fm, int mNumbOfTabsumb) {
            super(fm);
            this.NumbOfTabs = mNumbOfTabsumb;
            List<CharSequence> listItems = new ArrayList<>();
            listItems.add("Tab1");
            listItems.add("Tab2");
            listItems.add("Tab3");
            this.Titles = listItems.toArray(new CharSequence[listItems.size()]);
        }

        @Override
        public Fragment getItem(int position) {

            if(position == 1)
                return new NewsFeed();
            else if(position == 0)
                return new Friends();
            else
                return new LocalAudioFilesFragment();
        }

        @Override
        public CharSequence getPageTitle(int position) {
            return Titles[position];
        }

        @Override
        public int getCount() {
            return NumbOfTabs;
        }

    }
}
