package tads.ufpr.pr.br.backstage;

import android.app.ProgressDialog;
import android.os.AsyncTask;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.util.Base64;

import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONObject;

import java.net.URLDecoder;
import java.security.KeyStore;
import java.util.ArrayList;
import java.util.List;

import Util.DatabaseHandler;
import Util.MySSLSocketFactory;

import static org.apache.http.params.HttpConnectionParams.setConnectionTimeout;
import static org.apache.http.params.HttpConnectionParams.setSoTimeout;

@SuppressWarnings("ALL")
public class MessagesActivity extends AppCompatActivity {

    List<Message> listMessage = new ArrayList<>();
    UsersListAdapter ulad;
    RecyclerView _recView;
    DatabaseHandler dbh;
    Usuario u;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_messages);

        u = ((AppController)this.getApplication()).getUsuario();
        dbh = new DatabaseHandler(this);

        Toolbar mToolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);
        setTitle("Mensagens");
        mToolbar.setTitleTextColor(ContextCompat.getColor(this, R.color.white));
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        _recView = (RecyclerView)findViewById(R.id.recyclerView);

        new FetchMessages().execute();
    }

    private class FetchMessages extends AsyncTask<Void, Void, Void> {

        ProgressDialog progressDialog;

        @Override
        protected void onPreExecute() {
            super.onPreExecute();

            progressDialog = new ProgressDialog(MessagesActivity.this, R.style.AppTheme_Dark_Dialog);
            progressDialog.setIndeterminate(true);
            progressDialog.setCancelable(false);
            progressDialog.setMessage(getResources().getString(R.string.msg_updating));
            progressDialog.show();

        }

        @Override
        protected void onPostExecute(Void aVoid) {
            super.onPostExecute(aVoid);

            List<Usuario> lUsers = new ArrayList<>();

            progressDialog.dismiss();
            progressDialog.cancel();

            if ( !listMessage.isEmpty() ){

                for ( Message m : listMessage ){

                    Usuario usuario = m.getUsuario();

                    if ( m.getTexto() != null )
                        usuario.setCidade(m.getTexto());

                    lUsers.add(usuario);

                }

                ulad = new UsersListAdapter(lUsers, u);
                ulad.setActivity(MessagesActivity.this);
                _recView.setAdapter(ulad);

                LinearLayoutManager llm = new LinearLayoutManager(MessagesActivity.this);
                _recView.setLayoutManager(llm);
                _recView.setHasFixedSize(true);

            }
        }

        @Override
        protected Void doInBackground(Void... params) {

            try{

                KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
                trustStore.load(null, null);

                MySSLSocketFactory sf = new MySSLSocketFactory(trustStore);
                sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);

                BasicHttpParams httpParams = new BasicHttpParams();
                setConnectionTimeout(httpParams, 3000);
                setSoTimeout(httpParams, 4000);

                HttpProtocolParams.setVersion(httpParams, HttpVersion.HTTP_1_1);
                HttpProtocolParams.setContentCharset(httpParams, HTTP.UTF_8);

                SchemeRegistry registry = new SchemeRegistry();
                registry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
                registry.register(new Scheme("https", sf, 443));

                ClientConnectionManager ccm = new ThreadSafeClientConnManager(httpParams, registry);

                HttpClient httpclient = new DefaultHttpClient(ccm, httpParams);

                httpclient = new DefaultHttpClient(httpParams);

                String url = getApplicationContext().getString(R.string.server_address)
                        + "/usuario/" + u.getId() + "/chat/listar";

                HttpGet httpGet = new HttpGet(url);

                String suser = getApplicationContext().getString(R.string.server_user);
                String basicAuth = "Basic " + Base64.encodeToString(suser.getBytes(), Base64.NO_WRAP);
                httpGet.setHeader("Authorization", basicAuth);

                HttpResponse response = httpclient.execute(httpGet);
                if ( response != null && response.getStatusLine().getStatusCode() == 200 ){

                    String ret = URLDecoder.decode(EntityUtils.toString(response.getEntity()), HTTP.UTF_8);
                    JSONArray ja = new JSONArray(ret);

                    if ( ja.length() > 0 ){

                        for ( int i = 0 ; i < ja.length() ; i++ ){

                            JSONObject jo = ja.getJSONObject(i);

                            Message message = new Message();

                            String s = jo.get("id_usuarios").toString().replace("[", "");
                            s = s.replace("]", "");
                            int id = Integer.parseInt(s);

                            message.setChatId(jo.getInt("chat_id"));
                            message.setMsgId(id);

                            message.setTexto(dbh.getLastChatMessage(message.getChatId()));
                            listMessage.add(message);

                        }

                        response.getEntity().consumeContent();
                        httpGet.abort();

                        int i = 0;

                        for ( Message m : listMessage ){

                            Usuario usuario = dbh.getUserMessage(m.getMsgId());

                            if ( usuario == null ){

                                //Buscar e inserir na tabela de users
                                url = getString(R.string.server_address)
                                        + "/usuario/" + m.getMsgId();

                                httpGet = new HttpGet(url);
                                httpGet.setHeader("Authorization", basicAuth);
                                response = httpclient.execute(httpGet);

                                if ( response != null && response.getStatusLine().getStatusCode() == 200 ){

                                    ret = URLDecoder.decode(EntityUtils.toString(response.getEntity()), HTTP.UTF_8);
                                    ja = new JSONArray(ret);

                                    if ( ja != null ){

                                        JSONObject jo = ja.getJSONObject(0);

                                        usuario = new Usuario();
                                        usuario.setId(jo.getInt("id"));
                                        usuario.setNome(new String(jo.getString("nome").getBytes("ISO-8859-1"), HTTP.UTF_8));
                                        usuario.setImagem(new String(jo.getString("imagem").getBytes("ISO-8859-1"), HTTP.UTF_8).replace(" ", "+"));
                                        usuario.setCidade(new String(jo.getString("cidade").getBytes("ISO-8859-1"), HTTP.UTF_8));

                                        listMessage.get(i).setUsuario(usuario);
                                        dbh.setUserMessage(usuario);

                                    }

                                    response.getEntity().consumeContent();

                                }

                                response.getEntity().getContent().close();
                                httpGet.abort();

                            }
                            else {

                                listMessage.get(i).setUsuario(usuario);

                            }

                            i++;

                        }

                        httpclient.getConnectionManager().closeExpiredConnections();
                        httpclient.getConnectionManager().shutdown();

                    }
                }
            }
            catch (Exception e) { e.printStackTrace(); }

            return null;
        }
    }
}
