package tads.ufpr.pr.br.backstage;

import android.content.Context;
import android.graphics.Typeface;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewStub;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.List;

public class ProfileFragment extends Fragment {

    boolean isUser;

    public static ProfileFragment createInstance (Usuario user){
        ProfileFragment pf = new ProfileFragment();
        Bundle bundle = new Bundle();
        bundle.putSerializable("usuario", user);
        pf.setArguments(bundle);
        return pf;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        RecyclerView recyclerView = (RecyclerView) inflater.inflate(R.layout.profile_fragment, container, false);
        setupRecyclerView(recyclerView);
        return recyclerView;
    }

    private void setupRecyclerView(RecyclerView recyclerView) {
        recyclerView.setLayoutManager(new LinearLayoutManager(getActivity()));
        ProfileRecyclerAdapter pra = new ProfileRecyclerAdapter(createItensList());
        recyclerView.setAdapter(pra);
    }

    private List<Object> createItensList(){
        List<Object> lo = new ArrayList<>();
        Bundle bundle = getArguments();
        if(bundle != null){
            lo.add(bundle.getSerializable("usuario"));
        }
        return lo;
    }

    public class ProfileRecyclerAdapter extends RecyclerView.Adapter<ProfileRecyclerAdapter.InstViewHolder> {

        private List<Object> items;

        public class InstViewHolder extends RecyclerView.ViewHolder {

            LinearLayout ll_profile;
            ViewStub     stub_instrumentos;
            ViewStub     stub_profile;
            ViewStub     stub_style;

            InstViewHolder(View itemView){
                super(itemView);

                ll_profile        = (LinearLayout)itemView.findViewById(R.id.ll_profile);
                stub_profile      = (ViewStub)itemView.findViewById(R.id.stub_profile);
                stub_instrumentos = (ViewStub)itemView.findViewById(R.id.stub_instrumentos);
                stub_style        = (ViewStub)itemView.findViewById(R.id.stub_style_influence);

            }
        }

        public ProfileRecyclerAdapter(List<Object> items) {
            this.items = items;
        }

        @Override
        public ProfileRecyclerAdapter.InstViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.profile_viewholder, parent, false);
            InstViewHolder ivh = new InstViewHolder(view);
            return ivh;
        }

        @Override
        public void onBindViewHolder(InstViewHolder ivh, int position) {

            Usuario usuario = (Usuario)items.get(0);
            Typeface tf  = Typeface.createFromAsset(getActivity().getAssets(), "GeosansLight.ttf");

            if( usuario != null ){

                ViewStub stubProfile = ivh.stub_profile;
                stubProfile.setLayoutResource(R.layout.profile_import);
                View vsProfile = stubProfile.inflate();

                if ( usuario.getCidade() != null && !usuario.getCidade().isEmpty() )
                    ((TextView) vsProfile.findViewById(R.id.tv_cityName)).setText(usuario.getCidade());

                else
                    ((TextView) vsProfile.findViewById(R.id.tv_cityName)).setText(getString(R.string.city));

                if ( usuario.getEstado() != null && !usuario.getEstado().isEmpty()
                        && usuario.getPais() != null && !usuario.getPais().isEmpty())
                    ((TextView) vsProfile.findViewById(R.id.tv_stateCountry)).setText(usuario.getEstado() + ", " + usuario.getPais());

                else if (usuario.getEstado() != null && !usuario.getEstado().isEmpty())
                    ((TextView) vsProfile.findViewById(R.id.tv_stateCountry)).setText(usuario.getEstado() + ", " + getString(R.string.country));

                else if (usuario.getPais() != null && !usuario.getPais().isEmpty())
                    ((TextView) vsProfile.findViewById(R.id.tv_stateCountry)).setText(getString(R.string.state) + ", " + usuario.getPais());

                else
                    ((TextView) vsProfile.findViewById(R.id.tv_stateCountry)).setText(getString(R.string.state) + ", " + getString(R.string.country));

                if (usuario.getSobre() != null && !usuario.getSobre().isEmpty())
                    ((TextView) vsProfile.findViewById(R.id.tv_aboutMeText)).setText(usuario.getSobre());
                else
                    ((TextView) vsProfile.findViewById(R.id.tv_aboutMeText)).setText(getString(R.string.about_me_text));


                ViewStub stubInst = ivh.stub_instrumentos;
                stubInst.setLayoutResource(R.layout.instrumentos_import);
                View vsInst = stubInst.inflate();
                TextView no_inst = (TextView)vsInst.findViewById(R.id.no_inst_text);

                ListView lv = (ListView) vsInst.findViewById(R.id.instList);

                if( usuario.getInstrumentos() != null && !usuario.getInstrumentos().isEmpty() ){
                    AdapterListView alv = new AdapterListView(getActivity(), usuario.getInstrumentos());
                    lv.setAdapter(alv);
                    resizeList(lv);
                    no_inst.setVisibility(View.INVISIBLE);
                }
                else
                    no_inst.setText(getString(R.string.no_inst_error));

                ViewStub stubStyle = ivh.stub_style;
                stubStyle.setLayoutResource(R.layout.style_influence_import);
                View vsStyle = stubStyle.inflate();

                TextView styles = (TextView)vsStyle.findViewById(R.id.tv_styleText);
                TextView infl   = (TextView)vsStyle.findViewById(R.id.tv_inflText);

                if ( usuario.getInfluencias() != null && !usuario.getInfluencias().isEmpty() ){
                    for ( String s : usuario.getInfluencias() )
                        infl.setText(infl.getText() + s + "\n");
                }
                else
                    infl.setText(getString(R.string.no_infl_error));

                if ( usuario.getEstilos() != null && !usuario.getEstilos().isEmpty() ){
                    for ( String s : usuario.getEstilos() )
                        styles.setText(styles.getText() + s + "\n");
                }
                else
                    styles.setText(getString(R.string.no_styles_error));

                ((TextView)vsProfile.findViewById(R.id.profileTitle)).setTypeface(tf);
                ((TextView)vsInst.findViewById(R.id.instTitle)).setTypeface(tf);
                ((TextView)vsStyle.findViewById(R.id.infTitle)).setTypeface(tf);

                Bundle extras = getActivity().getIntent().getExtras();
                if (extras != null) {
                    isUser = extras.getBoolean("isUser");
                    if (!isUser){
                        (vsProfile.findViewById(R.id.edit_img)).setVisibility(View.INVISIBLE);
                        (vsStyle.findViewById(R.id.edit_img)).setVisibility(View.INVISIBLE);
                        (vsInst.findViewById(R.id.edit_img)).setVisibility(View.INVISIBLE);
                    }
                    else{
                        (vsProfile.findViewById(R.id.edit_img)).setVisibility(View.VISIBLE);
                        (vsStyle.findViewById(R.id.edit_img)).setVisibility(View.VISIBLE);
                        (vsInst.findViewById(R.id.edit_img)).setVisibility(View.VISIBLE);
                    }
                }
                else{
                    (vsProfile.findViewById(R.id.edit_img)).setVisibility(View.INVISIBLE);
                    (vsStyle.findViewById(R.id.edit_img)).setVisibility(View.INVISIBLE);
                    (vsInst.findViewById(R.id.edit_img)).setVisibility(View.INVISIBLE);
                }
            }
        }

        @Override
        public int getItemCount() { return this.items.size(); }

        @Override
        public void onAttachedToRecyclerView(RecyclerView recyclerView) {
            super.onAttachedToRecyclerView(recyclerView);
        }
    }

    private void resizeList(ListView lv) {

        AdapterListView alv = (AdapterListView) lv.getAdapter();
        if(alv != null){

            int numberOfItems    = alv.getCount();
            int totalItemsHeight = 0;

            for(int i = 0; i < numberOfItems ; i++){
                View item = alv.getView(i, null, lv);
                item.measure(0, 0);
                totalItemsHeight += item.getMeasuredHeight();
            }

            int totalDividersHeight = lv.getDividerHeight() * (numberOfItems - 1);

            ViewGroup.LayoutParams params = lv.getLayoutParams();
            params.height = totalItemsHeight + totalDividersHeight;
            lv.setLayoutParams(params);
            lv.requestLayout();
        }
    }

    public class AdapterListView extends BaseAdapter {

        private LayoutInflater mInflater;
        private List<Instrumento> itens;

        public AdapterListView(Context context, List<Instrumento> itens) {
            this.itens = itens;
            mInflater = LayoutInflater.from(context);
        }

        @Override
        public int getCount() {
            return itens.size();
        }

        @Override
        public Object getItem(int position) {
            return itens.get(position);
        }

        @Override
        public long getItemId(int position) {
            return position;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {

            ItemSuporte itemHolder;
            List<InstType> linstType = ((AppController)getActivity().getApplication()).getListInstType();

            if (convertView == null) {
                convertView = mInflater.inflate(R.layout.inst_list_item, null);
                itemHolder = new ItemSuporte();
                itemHolder.iv_InstType  = ((ImageView) convertView.findViewById(R.id.iv_insttype_logo));
                itemHolder.tv_InstName  = ((TextView) convertView.findViewById(R.id.tv_instName));
                itemHolder.tv_InstLevel = ((TextView) convertView.findViewById(R.id.tv_instLevel));
                convertView.setTag(itemHolder);
            }
            else{
                itemHolder = (ItemSuporte) convertView.getTag();
            }

            Instrumento item = itens.get(position);
            String[] array  = getResources().getStringArray(R.array.level_array);

            switch ( item.getInstType() ){

                case 1: //Cordas

                    itemHolder.iv_InstType.setImageResource(R.drawable.chords);
                    break;

                case 2: //Percussão

                    itemHolder.iv_InstType.setImageResource(R.drawable.drums);
                    break;

                case 3: //Sopro

                    itemHolder.iv_InstType.setImageResource(R.drawable.sax);
                    break;

                case 4: //Teclas

                    itemHolder.iv_InstType.setImageResource(R.drawable.teclado);
                    break;

                case 5: //Vocal

                    itemHolder.iv_InstType.setImageResource(R.drawable.mic);
                    break;
            }

            CharSequence cs = "";
            for ( Instrumento i : linstType.get( item.getInstType() - 1 ).getInstrumentos() ){
                if ( i.getInstId() == item.getInstId() ) {
                    cs = i.getInstName();
                    break;
                }
            }

            itemHolder.tv_InstName.setText(cs);
            itemHolder.tv_InstLevel.setText(array[item.getInstLevel()]);

            return convertView;
        }

        private class ItemSuporte {
            ImageView iv_InstType;
            TextView tv_InstName;
            TextView tv_InstLevel;
        }
    }
}
