package tads.ufpr.pr.br.backstage;

import android.content.Context;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.MediaRecorder;
import android.os.Build;
import android.os.Environment;
import android.os.SystemClock;
import android.os.Vibrator;
import android.support.design.widget.FloatingActionButton;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.View;
import android.widget.Chronometer;
import java.io.File;
import java.io.IOException;

public class RecordActivity extends AppCompatActivity {

    FloatingActionButton fab;

    static boolean inactive;
    static boolean recording;
    static boolean stoped;
    static boolean playing;

    String fpath;
    Chronometer _progressClock;
    private MediaRecorder mRecorder = null;
    private MediaPlayer   mPlayer   = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_record);

        DisplayMetrics dm = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(dm);
        getWindow().setLayout((int) (dm.widthPixels * .85), (int) (dm.heightPixels * .75));

        inactive = true;
        setVolumeControlStream(AudioManager.STREAM_MUSIC);
        _progressClock = (Chronometer)findViewById(R.id.progress_clock);

        mPlayer = new MediaPlayer();

        fab = (FloatingActionButton)findViewById(R.id.rec);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                if (inactive)
                    record();

                else if (recording)
                    stopRecord();

                else if (stoped)
                    play();

                else if (playing)
                    stopPlaying();

                Vibrator vb = (Vibrator) RecordActivity.this.getSystemService(Context.VIBRATOR_SERVICE);
                vb.vibrate(100);
            }
        });


        mPlayer.setOnCompletionListener(new MediaPlayer.OnCompletionListener() {
            public void onCompletion(MediaPlayer mediaPlayer) {

                stoped = true;
                playing = false;

                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
                    fab.setImageDrawable(getDrawable(R.drawable.play));
                else
                    fab.setImageDrawable(getResources().getDrawable(R.drawable.play));

            }
        });
    }

    private void stopPlaying() {

        stoped = true;
        playing = false;

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
            fab.setImageDrawable(getDrawable(R.drawable.play));
        else
            fab.setImageDrawable(getResources().getDrawable(R.drawable.play));

        mPlayer.release();
    }


    private void play() {

        stoped = false;
        playing = true;

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
            fab.setImageDrawable(getDrawable(R.drawable.stop));
        else
            fab.setImageDrawable(getResources().getDrawable(R.drawable.stop));

        try {
            if ( mPlayer == null )
                mPlayer = new MediaPlayer();

            mPlayer.setDataSource(fpath);
            mPlayer.prepare();
            mPlayer.start();

        } catch (IOException e) {
            Log.e("Player", "prepare() failed");
        }
    }

    private void stopRecord() {

        _progressClock.stop();
        mRecorder.stop();

        recording = false;
        stoped = true;

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
            fab.setImageDrawable(getDrawable(R.drawable.play));
        else
            fab.setImageDrawable(getResources().getDrawable(R.drawable.play));

    }

    public void record(){

        inactive  = false;
        recording = true;

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
            fab.setImageDrawable(getDrawable(R.drawable.stop));
        else
            fab.setImageDrawable(getResources().getDrawable(R.drawable.stop));

        try {

            File path = new File( Environment.getExternalStorageDirectory().getAbsolutePath()
                    + getString(R.string.filepath));
            path.mkdirs();
            path = File.createTempFile("temp_", ".mp3", path);

            fpath = Environment.getExternalStorageDirectory().getAbsolutePath()
                    + getString(R.string.filepath) + path.getName();

            path.delete();

            mRecorder = new MediaRecorder();
            mRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);
            mRecorder.setOutputFormat(MediaRecorder.OutputFormat.MPEG_4);
            mRecorder.setOutputFile(fpath);
            mRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.HE_AAC);
            mRecorder.prepare();

            _progressClock.setBase(SystemClock.elapsedRealtime());
            _progressClock.start();

            mRecorder.start();

        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    @Override
    protected void onPause() {
        super.onPause();

        if ( isFinishing() && mPlayer != null) {
            mPlayer.release();
            mPlayer = null;
        }
    }
}
