package tads.ufpr.pr.br.backstage;

import android.app.Fragment;
import android.content.Intent;
import android.graphics.Typeface;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.design.widget.FloatingActionButton;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.CompoundButton;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.Spinner;
import android.widget.Switch;
import android.widget.TextView;
import java.util.ArrayList;
import java.util.List;
import butterknife.ButterKnife;
import butterknife.InjectView;

public class SearchFilterFragment  extends Fragment {

    @InjectView(R.id.sw_inst)       Switch               _swInst;
    @InjectView(R.id.sb_dist)       SeekBar              _skDist;
    @InjectView(R.id.kmval)         TextView             _kmView;
    @InjectView(R.id.instType_sp)   Spinner              _spType;
    @InjectView(R.id.instrument_sp) Spinner              _spInst;
    @InjectView(R.id.level_sp)      Spinner              _spLevel;
    @InjectView(R.id.inst_filter)   LinearLayout         _instFilter;
    @InjectView(R.id.fabButton)     FloatingActionButton _fabButton;

    View v;
    List<InstType> instTypeList;

    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container,
                             @Nullable Bundle savedInstanceState) {

        v = inflater.inflate(R.layout.search_filter_fragment, container, false);
        ((AppController)getActivity().getApplication()).instantiateInstList(getActivity());
        ButterKnife.inject(this, v);

        Typeface tf = Typeface.createFromAsset(getActivity().getAssets(), "GeosansLight.ttf");
        ((TextView) v.findViewById(R.id.search_by_dist_title)).setTypeface(tf);
        ((TextView) v.findViewById(R.id.search_by_inst_title)).setTypeface(tf);

        distOptionsHandler();

        _swInst.setOnCheckedChangeListener(swListener);

        initInstrumentList();

        _fabButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                searchButtonHandler();
            }
        });

        return v;
    }

    private void distOptionsHandler() {

        _skDist.setMax(501);
        _skDist.setProgress(20);
        _kmView.setText(getString(R.string.till) + " " + 20 + " " + getString(R.string.km));

        _skDist.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {

            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (progress <= 0) {
                    _kmView.setText(getString(R.string.till) + " " + 500 + " metros");
                } else if (progress > 500)
                    _kmView.setText("Sem limite");
                else {
                    _kmView.setText(getString(R.string.till) + " " + progress + " " + getString(R.string.km));
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) { }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) { }
        });
    }

    private void initInstrumentList() {

        instTypeList = ((AppController)getActivity().getApplication()).getListInstType();

        List<String> instList = new ArrayList<>();
        for(Instrumento i : instTypeList.get(0).getInstrumentos())
            instList.add(i.getInstName());

        InstTypeSpinnerAdapter itsa = new InstTypeSpinnerAdapter(getActivity(), R.id.rl_spinner_instType, R.id.tv_instTypeName, instTypeList);
        _spType.setAdapter(itsa);

        ArrayAdapter<String> spArrayAdapter = new ArrayAdapter<>(getActivity(), android.R.layout.simple_spinner_item, instList);
        spArrayAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        _spInst.setAdapter(spArrayAdapter);

        String[] arrayLevel = getResources().getStringArray(R.array.level_array);
        List<String> levelList = new ArrayList<>();
        for (String s : arrayLevel)
            levelList.add(s);

        spArrayAdapter = new ArrayAdapter<>(getActivity(), android.R.layout.simple_spinner_item, levelList);
        spArrayAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        _spLevel.setAdapter(spArrayAdapter);


        _spType.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {

            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {

                List<String> list = new ArrayList<>();
                for ( Instrumento i : instTypeList.get(position).getInstrumentos()) {
                    String s = i.getInstName();
                    list.add(s);
                }
                ArrayAdapter<String> spinnerArrayAdapter = new ArrayAdapter<>(getActivity(), android.R.layout.simple_spinner_item, list);
                spinnerArrayAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
                _spInst.setAdapter(spinnerArrayAdapter);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) { }
        });

    }

    private void searchButtonHandler() {

        String text = ((TextView)getActivity().findViewById(R.id.searchText)).getText().toString();

        //distância em metros
        int dist = _skDist.getProgress() == 0 ? 500 : _skDist.getProgress() * 1000 ;

        if ( !text.isEmpty() && !_swInst.isChecked() ){

            //Busca só por texto
            Intent i = new Intent(getActivity(), ResultSearchActivity.class);
            i.putExtra("option", 1);
            i.putExtra("dist", dist);
            i.putExtra("text", text);
            startActivity(i);

        }
        else if  ( text.isEmpty() && _swInst.isChecked() ){

            //Busca só por instrumento
            Intent i = new Intent(getActivity(), ResultSearchActivity.class);
            i.putExtra("option", 2);
            i.putExtra("type",  instTypeList.get(_spType.getSelectedItemPosition()).getTypeId());
            i.putExtra("inst",  instTypeList.get(_spType.getSelectedItemPosition())
                    .getInstrumentos().get(_spInst.getSelectedItemPosition()));
            i.putExtra("level", _spLevel.getSelectedItemPosition());
            startActivity(i);

        }
        else if ( !text.isEmpty() && _swInst.isChecked() ){

            //Busca por texto e instrumento
            Intent i = new Intent(getActivity(), ResultSearchActivity.class);
            i.putExtra("option", 3);
            i.putExtra("text", text);
            i.putExtra("type",  instTypeList.get(_spType.getSelectedItemPosition()).getTypeId());
            i.putExtra("inst",  instTypeList.get(_spType.getSelectedItemPosition())
                    .getInstrumentos().get(_spInst.getSelectedItemPosition()));
            i.putExtra("level", _spLevel.getSelectedItemPosition());
            startActivity(i);
        }
        else{

            //Busca por distância
            Intent i = new Intent(getActivity(), ResultSearchActivity.class);
            i.putExtra("option", 4);
            i.putExtra("dist", dist);
            startActivity(i);

        }
    }

    CompoundButton.OnCheckedChangeListener swListener = new CompoundButton.OnCheckedChangeListener(){

        @Override
        public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {

            if ( _instFilter.getVisibility() == View.GONE ) {

                _instFilter.setVisibility(View.INVISIBLE);
                Animation animFadeIn = AnimationUtils.loadAnimation(
                        getActivity().getApplicationContext(), android.R.anim.fade_in);

                animFadeIn.setDuration(600);
                _instFilter.startAnimation(animFadeIn);
                _instFilter.setVisibility(View.VISIBLE);

            }
            else {

                Animation animFadeOut = AnimationUtils.loadAnimation(
                        getActivity().getApplicationContext(), android.R.anim.fade_out);

                animFadeOut.setDuration(400);
                _instFilter.startAnimation(animFadeOut);
                _instFilter.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        _instFilter.setVisibility(View.GONE);
                    }
                }, 390);
            }

        }
    };
}
