package tads.ufpr.pr.br.backstage;

import android.app.ProgressDialog;
import android.content.Intent;
import android.graphics.Typeface;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.util.Base64;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import java.net.URLDecoder;
import java.security.KeyStore;
import java.util.ArrayList;
import java.util.List;

import Util.DatabaseHandler;
import Util.MySSLSocketFactory;
import Util.Utils;
import butterknife.ButterKnife;
import butterknife.InjectView;

import static org.apache.http.params.HttpConnectionParams.setConnectionTimeout;
import static org.apache.http.params.HttpConnectionParams.setSoTimeout;

@SuppressWarnings("All")
public class SignIn extends Fragment {

    @InjectView(R.id.btn_login)         Button          _loginButton;
    @InjectView(R.id.input_email)       EditText        _emailText;
    @InjectView(R.id.input_password)    EditText        _passwordText;
    ProgressDialog progressDialog;
    Usuario u;

    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {


        View v = inflater.inflate(R.layout.signin_fragment_layout, container, false);

        ButterKnife.inject(this, v);

        Typeface tf  = Typeface.createFromAsset(getActivity().getAssets(), "GeosansLight.ttf");
        ((TextView)v.findViewById(R.id.app_name)).setTypeface(tf);

        _loginButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                login();
            }
        });

        return v;
    }

    public void login() {

        _loginButton.setEnabled(false);

        if (!validate()) {
            onLoginFailed();
            return;
        }

        u = new Usuario();
        u.setEmail(_emailText.getText().toString());
        u.setSenha(_passwordText.getText().toString());

        if (Utils.isNetworkOnline(this.getContext())) {

            new doSignIn().execute();

        }
        else{
            //Sem Conexão com a Internet
            Utils.alertDialog(getActivity(), getResources().getString(R.string.no_network_title),
                    getResources().getString(R.string.no_network_text), 1);
            _loginButton.setEnabled(true);
            return;
        }
    }

    public void onLoginFailed() {
        _loginButton.setEnabled(true);
    }

    private boolean validate() {

        boolean valid = true;

        String email    = _emailText.getText().toString();
        String password = _passwordText.getText().toString();

        if ( email.isEmpty() || !android.util.Patterns.EMAIL_ADDRESS.matcher(email).matches()) {
            _emailText.requestFocus();
            _emailText.setError(getResources().getString(R.string.wrong_email_error));
            valid = false;
        }
        else
            _emailText.setError(null);

        if (password.isEmpty() || password.length() < 4 ) {
            _passwordText.requestFocus();
            _passwordText.setError(getResources().getString(R.string.few_chars_pass_error));
            valid = false;
        }
        else
            _passwordText.setError(null);

        return valid;
    }

    private class doSignIn extends AsyncTask<Void, Void, Void> {

        boolean logged;
        HttpResponse response;
        String ret;

        @Override
        protected void onPreExecute() {
            super.onPreExecute();

            progressDialog = new ProgressDialog(getActivity(), R.style.AppTheme_Dark_Dialog);
            progressDialog.setIndeterminate(true);
            progressDialog.setCancelable(false);
            progressDialog.setMessage(getResources().getString(R.string.authenticating));
            progressDialog.show();

        }

        @Override
        protected void onPostExecute(Void aVoid) {
            super.onPostExecute(aVoid);

            progressDialog.dismiss();
            progressDialog.cancel();
            _loginButton.setEnabled(true);

            if ( logged ){

                DatabaseHandler db = new DatabaseHandler(getActivity());
                db.addUser(u);

                ((AppController) getActivity().getApplication()).setUsuario(u);
                ((AppController)getActivity().getApplication()).instantiateInstList(getActivity());

                //Intent iservice = new Intent(getActivity(), GetMessages.class);
                //getActivity().startService(iservice);

                Intent i = new Intent(getActivity(), MainActivity.class);
                startActivity(i);
                getActivity().finish();

            }
            else {

                Utils.alertDialog(getActivity(), getResources().getString(R.string.login_error_title),
                        getResources().getString(R.string.login_error_text), 1);

            }
        }

        @Override
        protected Void doInBackground(Void... params) {

            try{

                KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
                trustStore.load(null, null);

                MySSLSocketFactory sf = new MySSLSocketFactory(trustStore);
                sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);

                BasicHttpParams httpParams = new BasicHttpParams();
                setConnectionTimeout(httpParams, 3000);
                setSoTimeout(httpParams, 5000);

                HttpProtocolParams.setVersion(httpParams, HttpVersion.HTTP_1_1);
                HttpProtocolParams.setContentCharset(httpParams, HTTP.UTF_8);

                SchemeRegistry registry = new SchemeRegistry();
                registry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
                registry.register(new Scheme("https", sf, 443));

                ClientConnectionManager ccm = new ThreadSafeClientConnManager(httpParams, registry);

                HttpClient httpclient = new DefaultHttpClient(ccm, httpParams);

                httpclient = new DefaultHttpClient(httpParams);

                String url = getString( R.string.server_address ) + "/login";
                HttpPost httppost = new HttpPost(url);

                List<NameValuePair> lp = new ArrayList<>();
                lp.add(new BasicNameValuePair("mail", u.getEmail()));

                String md5Hex = Utils.getMd5Hash(u.getSenha());

                lp.add(new BasicNameValuePair("pass", md5Hex));

                String user = getResources().getString(R.string.server_user);
                String basicAuth = "Basic " + Base64.encodeToString(user.getBytes(), Base64.NO_WRAP);
                httppost.setHeader("Authorization", basicAuth);

                UrlEncodedFormEntity urle = new UrlEncodedFormEntity(lp, HTTP.UTF_8);
                httppost.setEntity(urle);
                response = httpclient.execute(httppost);

                if ( response != null && response.getStatusLine().getStatusCode() == 200 ) {

                    ret = URLDecoder.decode(EntityUtils.toString(response.getEntity()), HTTP.UTF_8);
                    logged = true;

                    JSONArray ja = new JSONArray(ret);

                    if ( ja != null ){

                        httppost.getEntity().consumeContent();
                        httppost.abort();

                        JSONObject jo = ja.getJSONObject(0);
                        u.setId(Integer.parseInt(jo.getString("id")));
                        //u.setEmail(new String(jo.getString("email").getBytes("ISO-8859-1"), HTTP.UTF_8));
                        u.setNome(new String(jo.getString("nome").getBytes("ISO-8859-1"), HTTP.UTF_8));

                        String sobre = new String(jo.getString("sobre").getBytes("ISO-8859-1"), HTTP.UTF_8);
                        if ( !sobre.isEmpty() && !sobre.equals("null") )
                            u.setSobre(sobre);

                        u.setDtNasc(new String(jo.getString("data_nasc").getBytes("ISO-8859-1"), HTTP.UTF_8));
                        u.setLatitude(Double.parseDouble(jo.getString("latitude")));
                        u.setLongitude(Double.parseDouble(jo.getString("longitude")));
                        u.setCidade(new String(jo.getString("cidade").getBytes("ISO-8859-1"), HTTP.UTF_8));
                        u.setEstado(new String(jo.getString("estado").getBytes("ISO-8859-1"), HTTP.UTF_8));
                        u.setPais(new String(jo.getString("pais").getBytes("ISO-8859-1"), HTTP.UTF_8));
                        u.setImagem(new String(jo.getString("imagem").getBytes("ISO-8859-1"), HTTP.UTF_8).replace(" ", "+"));

                        List<String> ls = new ArrayList<>();
                        String[] estilos = new String(jo.getString("estilos").getBytes("ISO-8859-1"), HTTP.UTF_8).split(",");
                        for ( String s : estilos ){
                            if ( !s.isEmpty() && !s.equals("null") )
                                ls.add( s  );
                        }
                        u.setEstilos(ls);

                        ls = new ArrayList<>();
                        String[] influ = new String(jo.getString("influencias").getBytes("ISO-8859-1"), HTTP.UTF_8).split(",");
                        for ( String s : influ ) {
                            if ( !s.isEmpty() && !s.equals("null") )
                                ls.add( s  );
                        }
                        u.setInfluencias(ls);

                        url = getString( R.string.server_address )
                                + "/usuario/" + u.getId() + "/instrumentos" ;

                        HttpGet httpGet = new HttpGet(url);
                        httpGet.setHeader("Authorization", basicAuth);

                        response = httpclient.execute(httpGet);

                        if ( response != null && response.getStatusLine().getStatusCode() == 200 ) {

                            ret = URLDecoder.decode(EntityUtils.toString(response.getEntity()), HTTP.UTF_8);

                            if ( !ret.isEmpty() ){

                                ja = new JSONArray(ret);

                                if ( ja != null ){

                                    List<Instrumento> lInst = new ArrayList<>();

                                    for ( int i = 0 ; i < ja.length() ; i++ ){

                                        jo = ja.getJSONObject(i);
                                        Instrumento instrumento = new Instrumento();
                                        instrumento.setInstId(jo.getInt("id"));
                                        instrumento.setInstName(new String(jo.getString("nome").getBytes("ISO-8859-1"), HTTP.UTF_8));
                                        instrumento.setInstType(jo.getInt("tipo_instrumento_id"));
                                        instrumento.setInstLevel(jo.getInt("nivel"));

                                        lInst.add(instrumento);

                                    }

                                    u.setInstrumentos(lInst);

                                }
                            }

                            response.getEntity().consumeContent();
                            httpGet.abort();

                        }


                        //Get Follows
                        url = getString( R.string.server_address ) + "/usuario/" + u.getId()
                                + "/seguir";

                        httpGet = new HttpGet(url);
                        httpGet.setHeader("Authorization", basicAuth);

                        response = httpclient.execute(httpGet);

                        if ( response != null && response.getStatusLine().getStatusCode() == 200 ){

                            ret = URLDecoder.decode(EntityUtils.toString(response.getEntity()), HTTP.UTF_8);

                            response.getEntity().consumeContent();

                            if ( !ret.isEmpty() ){

                                ja = new JSONArray(ret);

                                if ( ja != null ){

                                    DatabaseHandler databaseHandler = new DatabaseHandler(getActivity());

                                    for ( int i = 0 ; i < ja.length() ; i++ ){

                                        jo = ja.getJSONObject(i);

                                        url = getString(R.string.server_address) + "/usuario/"
                                                + jo.getInt("usuario_id_seguido");

                                        httpGet = new HttpGet(url);
                                        httpGet.setHeader("Authorization", basicAuth);

                                        response = httpclient.execute(httpGet);

                                        if ( response != null && response.getStatusLine().getStatusCode() == 200 ){

                                            ret = URLDecoder.decode(EntityUtils.toString(response.getEntity()), HTTP.UTF_8);

                                            response.getEntity().consumeContent();

                                            if ( !ret.isEmpty() ){

                                                JSONArray ja1 = new JSONArray(ret);
                                                jo = ja1.getJSONObject(0);

                                                Usuario usuario = new Usuario();

                                                usuario.setId(Integer.parseInt(jo.getString("id")));
                                                usuario.setNome(new String(jo.getString("nome").getBytes("ISO-8859-1"), HTTP.UTF_8));
                                                usuario.setImagem(new String(jo.getString("imagem").getBytes("ISO-8859-1"), HTTP.UTF_8).replace(" ", "+"));
                                                usuario.setCidade(new String(jo.getString("cidade").getBytes("ISO-8859-1"), HTTP.UTF_8));

                                                databaseHandler.addFriend(usuario);

                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception e)     { e.printStackTrace(); }

            return null;
        }


    }
}
