local method = ngx.req.get_method()

if method == "POST" or method == "PUT" then
	local upload = require "resty.upload"
	local r_sha1 = require "resty.sha1"
	local r_str  = require "resty.string"
	local cjson  = require "cjson"
	
	local chunk_size = 8192
	local upload_dir = ngx.var.dir
	
	local form, err = upload:new(chunk_size)
	
	if not form then
		ngx.log(ngx.ERR, "failed to new upload: ", err)
		ngx.status = ngx.HTTP_INTERNAL_SERVER_ERROR		-- 500
		ngx.say([[{"error": "Failed to new upload: "]] .. err ..[["}]])
		ngx.exit(ngx.status)	--Para totalmente o processamento do request.
	end
	
	form:set_timeout(1000)	-- 1 seconds
	
	local function is_empty(var)
		return var == nil or var == ""
	end
	
	local function get_file_name(line)
		local f_name = nil
		--for k, v in pairs(line) do
			--ngx.log(ngx.INFO, "## File: Key and Value: \r\n\t", k, " => ", v, "\r\n")
			ngx.log(ngx.INFO, "## FILE: Value: \r\n\t", line[3], "\r\n")
			--if not f_name then
				--if string.match(line[3], [[Content%-Disposition:.+filename=".+"]]) then
					--print("get_file_name ENTROU NO CONTENT")
					f_name = string.match(line[3], [[Content%-Disposition:.+filename="(.+)"]])
				--end
			--end
		--end
		if not f_name then
			ngx.log(ngx.INFO, "File name not found.")
			return nil
		else
			ngx.log(ngx.INFO, "File name FOUND: ", f_name)
			return f_name
		end
	end
	
	local function get_arg_name(line)
		local a_name = nil
		--for k, v in pairs(line) do
			--ngx.log(ngx.INFO, "## ARG: Key and Value: \r\n\t", k, " => ", v, "\r\n")
			ngx.log(ngx.INFO, "## ARG: Value: \r\n\t", line[3], "\r\n")
			--if not a_name then
				--if string.match(line[3], [[Content%-Disposition:.+name]]) then
					--print("get_arg_name ENTROU NO CONTENT")
					a_name = string.match(line[3], [[Content%-Disposition:.+name="(.+)"]])
				--end
			--end
		--end
		if not a_name then
			ngx.log(ngx.INFO, "Argument name not found.")
			return nil
		else
			ngx.log(ngx.INFO, "Arg name FOUND: ", a_name)
			return a_name
		end
	end
	
	local sha1 = r_sha1:new()
	local sha1_sum = nil
	local file = nil
	local orig_sha1 = nil
	local args = {}
	local arg_name = nil
	
	local flag_get_header = true
	
	while true do
		local token_type, res, err = form:read()
		
		if not token_type then
			ngx.log(ngx.ERR, "Failed to read: ", err)
			ngx.status = ngx.HTTP_BAD_REQUEST		-- 400
			ngx.say("Failed to read: ", err)
			return
		end
		
		ngx.log(ngx.INFO, "Reading: ", cjson.encode({token_type, res}))
		if token_type == "header" and flag_get_header then
			local file_name = get_file_name(res)
			if file_name == nil then
				if arg_name == nil then
					arg_name = get_arg_name(res)
					if arg_name ~= nil then
						flag_get_header = false
						--arg_name = string.gsub(arg_name, [==[^%"]==], "")
						--arg_name = string.gsub(arg_name, [==[%"$]==], "")
					end
				end
			else
				flag_get_header = false
				--file_name = string.gsub(file_name, [==[^%"]==], "")
				--file_name = string.gsub(file_name, [==[%"$]==], "")
				print("FILE PATH: ", upload_dir .. file_name)
				file = assert(io.open(upload_dir .. file_name, "w"))
				orig_sha1 = file_name
				if not file then
					ngx.status = ngx.HTTP_INTERNAL_SERVER_ERROR		-- 500
					ngx.say([[{"error":"Failed to open file ']] .. upload_dir .. file_name .. [[', ]] .. file .. [["}]])
					return
				end
			end
		
		elseif token_type == "body" then
			if file then
				file:write(res)
				local ok = sha1:update(res)
				if not ok then
					status = 500
					ngx.status = ngx.HTTP_INTERNAL_SERVER_ERROR		-- 500
					ngx.say([[{"error":"SHA1 failed to add data"]])
					return
				end
				print("Content written in file!")
			else
				if is_empty(res) then
					args[arg_name] = nil
				else
					if args[arg_name] then
						args[arg_name] = args[arg_name] .. res
						print("Content appended in existing ARG: ", arg_name)
					else
						args[arg_name] = res
						print("New ARG written: ", arg_name)
					end
				end
			end
		
		elseif token_type == "part_end" then
			flag_get_header = true
			if file then
				file:close()
				file = nil
				sha1_sum = sha1:final()
				sha1:reset()
				print("\r\n## SHA-1 sum     : ", r_str.to_hex(sha1_sum), "\r\n## SHA-1 received: ", orig_sha1, "\r\n\t")
			else
				ngx.log(ngx.INFO, "ARGS => ", cjson.encode({args}))
				if arg_name ~= "type" and args[arg_name] ~= nil then
					args[arg_name] = "'" .. args[arg_name] .. "'"
				end
				arg_name = nil
			end
		
		elseif token_type == "eof" then
			break
		
		else
			--ngx.log(ngx.INFO, "Fim da parte atual!")
		end
		
	end		-- Fim do while 1 --
	
	local token_type, res, err = form:read()
	ngx.log(ngx.INFO, "Read: ", cjson.encode({token_type, res}))
	if orig_sha1 and sha1_sum then
		args["SHA-1"] = r_str.to_hex(sha1_sum)
		if orig_sha1 == args["SHA-1"] then
			ngx.var.sha1 = "'" .. args["SHA-1"] .. "'"
			ngx.var.down_url = "/" .. upload_dir .. args["SHA-1"]
			if method == "POST" then
				ngx.status = ngx.HTTP_CREATED		-- 201
			else --if method == "PUT" then
				ngx.status = ngx.HTTP_OK		-- 200
			end
		else
			ngx.status = ngx.HTTP_INTERNAL_SERVER_ERROR		-- 500
			local json = [[{"error": "Integrity failed", "SHA-1_sum": "]] .. r_str.to_hex(sha1_sum) .. [[", "SHA-1_received": "]] .. orig_sha1 .. [["}]]
			ngx.say(json)
			return
		end
	end
	ngx.var.text = args["text"] or "NULL"
	ngx.var.link = args["link"] or "NULL"
	ngx.var.type = args["type"]
	print(cjson.encode({args}))
	return
else
	print("Não é POST nem PUT. Método: ", method)
	return
end
