--CREATE ROLE myband SUPERUSER LOGIN PASSWORD 'TCC_TADS';
DROP DATABASE IF EXISTS myband;
CREATE DATABASE myband ENCODING = 'UTF8' OWNER myband;
\connect myband myband;
--\connect myband myband localhost;
--\connect myband;

BEGIN;

CREATE EXTENSION postgis;


CREATE TABLE usuario (
	id          SERIAL PRIMARY KEY,
	email       VARCHAR(255) UNIQUE NOT NULL,
	senha	      VARCHAR(255) NOT NULL,
	nome	      VARCHAR(255) NOT NULL,
	imagem			TEXT,
	data_nasc   DATE,
	estilos	    TEXT,
	influencias TEXT,
	sobre	      TEXT,
	latitude    DOUBLE PRECISION,
	longitude   DOUBLE PRECISION,
	point       GEOMETRY(POINT, 4326),
	cidade 	    VARCHAR(255) NOT NULL,
	estado	    VARCHAR(255) NOT NULL,
	pais	      VARCHAR(255) NOT NULL,
	insert_ts   TIMESTAMP WITH TIME ZONE NOT NULL,
	update_ts   TIMESTAMP WITH TIME ZONE NOT NULL DEFAULT CURRENT_TIMESTAMP,
	delete_ts   TIMESTAMP WITH TIME ZONE,
	ativo       BOOLEAN DEFAULT 'True'
);

CREATE TABLE login (
	id         SERIAL PRIMARY KEY,
	usuario_id INTEGER NOT NULL,
	insert_ts  TIMESTAMP WITH TIME ZONE NOT NULL DEFAULT CURRENT_TIMESTAMP,
	api_key    VARCHAR(40),
	
	CONSTRAINT login_usuario_id_fkey FOREIGN KEY (usuario_id) REFERENCES usuario (id) ON DELETE CASCADE
);

CREATE TABLE tipo_instrumento (
	id   SERIAL PRIMARY KEY,
	tipo VARCHAR(255) NOT NULL
);

CREATE TABLE instrumento (
	id                     SERIAL PRIMARY KEY,
	tipo_instrumento_id    INTEGER NOT NULL,
	nome                   VARCHAR(255) NOT NULL,
	
	CONSTRAINT instrumento_tipo_instrumento_id_fkey FOREIGN KEY (tipo_instrumento_id) REFERENCES tipo_instrumento (id) ON DELETE RESTRICT
);

CREATE TABLE inst_usuario (
	usuario_id     INTEGER NOT NULL,
	instrumento_id INTEGER NOT NULL,
	nivel	         INTEGER NOT NULL,
	
	PRIMARY KEY (usuario_id, instrumento_id),
	
	CONSTRAINT instr_usuario_usuario_id_fkey FOREIGN KEY (usuario_id) REFERENCES usuario (id) ON DELETE CASCADE,
	CONSTRAINT instr_usuario_instrumento_id_fkey FOREIGN KEY (instrumento_id) REFERENCES instrumento (id) ON DELETE RESTRICT
);

CREATE TABLE tipo_postagem (
	id   INTEGER PRIMARY KEY,
	tipo VARCHAR(50) NOT NULL
);

CREATE TABLE postagem (
	id               SERIAL PRIMARY KEY,
	usuario_id       INTEGER NOT NULL,
	tipo_postagem_id INTEGER NOT NULL,
	texto            TEXT,
	link             VARCHAR(255),
	anexo            VARCHAR(255),
	insert_ts        TIMESTAMP WITH TIME ZONE NOT NULL DEFAULT CURRENT_TIMESTAMP,
	
	CONSTRAINT postagem_usuario_id_fkey FOREIGN KEY (usuario_id) REFERENCES usuario (id) ON DELETE CASCADE,
	CONSTRAINT postagem_tipo_postagem_id_fkey FOREIGN KEY (tipo_postagem_id) REFERENCES tipo_postagem (id) ON DELETE RESTRICT
);

CREATE TABLE audio (
	id           SERIAL PRIMARY KEY,
	postagem_id  INTEGER NOT NULL,
	artista      VARCHAR(255),
	album        VARCHAR(255),
	musica       VARCHAR(255),
	descricao    TEXT,
	
	CONSTRAINT audio_postagem_id_fkey FOREIGN KEY (postagem_id) REFERENCES postagem (id) ON DELETE CASCADE
);

CREATE TABLE seguir (
	usuario_id_seguidor  INTEGER NOT NULL,
	usuario_id_seguido   INTEGER NOT NULL,
	insert_ts            TIMESTAMP WITH TIME ZONE NOT NULL DEFAULT CURRENT_TIMESTAMP,
	
	PRIMARY KEY (usuario_id_seguidor, usuario_id_seguido),
	
	CONSTRAINT seguir_usuario_id_seguidor_fkey FOREIGN KEY (usuario_id_seguidor) REFERENCES usuario (id) ON DELETE RESTRICT,
	CONSTRAINT seguir_usuario_id_seguido_fkey  FOREIGN KEY (usuario_id_seguido)  REFERENCES usuario (id) ON DELETE RESTRICT,
	CONSTRAINT seguir_validar_chk CHECK (usuario_id_seguidor != usuario_id_seguido)
);

CREATE TABLE chat (
	id          SERIAL PRIMARY KEY,
	nome        VARCHAR(255),
	
	insert_ts   TIMESTAMP WITH TIME ZONE NOT NULL DEFAULT CURRENT_TIMESTAMP
);

CREATE TABLE chat_usuario (
	chat_id     INTEGER NOT NULL,
	usuario_id  INTEGER NOT NULL,
	insert_ts   TIMESTAMP WITH TIME ZONE NOT NULL DEFAULT CURRENT_TIMESTAMP,
	
	PRIMARY KEY (chat_id, usuario_id),
	
	CONSTRAINT chat_usuario_usuario_id_fkey FOREIGN KEY (usuario_id) REFERENCES usuario (id) ON DELETE RESTRICT,
	CONSTRAINT chat_usuario_chat_id_fkey FOREIGN KEY (chat_id) REFERENCES chat (id)
);

CREATE TABLE chat_mensagem (
	id          SERIAL PRIMARY KEY,
	usuario_id  INTEGER NOT NULL,
	chat_id     INTEGER NOT NULL,
	texto       TEXT NOT NULL,
	insert_ts   TIMESTAMP WITH TIME ZONE NOT NULL DEFAULT CURRENT_TIMESTAMP,
	
	CONSTRAINT chat_mensagem_usuario_id_fkey FOREIGN KEY (usuario_id) REFERENCES usuario (id) ON DELETE RESTRICT,
	CONSTRAINT chat_mensagem_chat_id_fkey FOREIGN KEY (chat_id) REFERENCES chat (id)
);


CREATE OR REPLACE FUNCTION new_chat(name VARCHAR(255), user_id_1 INTEGER, user_id_2 INTEGER) RETURNS INTEGER
AS $$
	DECLARE
		c_id INTEGER;	-- chat_id
	BEGIN
		SELECT chat_id INTO c_id
		FROM chat_usuario
		GROUP BY chat_id
		HAVING ARRAY[user_id_1, user_id_2] <@ array_agg(usuario_id);
		
		IF c_id IS NULL THEN
			WITH ins_chat AS (
				INSERT INTO chat (nome)
				VALUES (name)
				RETURNING id AS cid
			),
			ins_chat_user AS (
				INSERT INTO chat_usuario (chat_id, usuario_id)
				SELECT cid, user_id_1
				FROM   ins_chat
			)
			INSERT INTO chat_usuario (chat_id, usuario_id) SELECT cid, user_id_2 FROM ins_chat
			RETURNING chat_id INTO c_id;
		END IF;
		
		RETURN c_id;
	END;
$$ LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION key_gen ()	RETURNS text AS
	'select (random())::text;'
LANGUAGE SQL;

/*
CREATE TRIGGER make_api_key
	BEFORE INSERT
	ON login
	EXECUTE procedure key_gen()
;
*/


CREATE OR REPLACE FUNCTION set_insert_ts()
RETURNS TRIGGER AS $$
BEGIN
	NEW.insert_ts = now();
	RETURN NEW;
END;
$$ language 'plpgsql';


CREATE OR REPLACE FUNCTION set_point(lon float, lat float)
RETURNS GEOMETRY(POINT, 4326) AS $$
	SELECT ST_SetSRID(ST_Point($1, $2), 4326);
$$ language 'sql';


/*
CREATE OR REPLACE FUNCTION set_delete_ts()
RETURNS TRIGGER AS $$
BEGIN
	OLD.delete_ts = now();
	OLD.ativo     = 'False';
	RETURN NULL;
END;
$$ language 'plpgsql';


CREATE OR REPLACE FUNCTION set_update_ts()
RETURNS TRIGGER AS $$
BEGIN
	IF row(NEW.*) IS DISTINCT FROM row(OLD.*) THEN
		NEW.update_ts = now(); 
		RETURN NEW;
	ELSE
		RETURN OLD;
	END IF;
END;
$$ language 'plpgsql';
*/

CREATE TRIGGER trig_insert_ts_usuario BEFORE INSERT ON usuario FOR EACH ROW EXECUTE PROCEDURE set_insert_ts();

--CREATE TRIGGER trig_update_ts_usuario BEFORE INSERT ON usuario FOR EACH ROW EXECUTE PROCEDURE set_update_ts();

--CREATE TRIGGER trig_delete_ts_usuario BEFORE DELETE ON usuario FOR EACH ROW EXECUTE PROCEDURE set_delete_ts();


END;
